# -*- coding: utf-8 -*-
"""
link - Link 对象类型定义和扩展函数模块

对应 SAP2000 的 LinkObj 相关设置

注意: 荷载相关函数在 loads/link_load.py
注意: 属性定义（PropLink）在 section/link_section.py

用法:
    # AI Agent 可以通过分类索引发现功能
    from link import LINK_API_CATEGORIES
    
    # 查看局部轴相关函数
    print(LINK_API_CATEGORIES["局部坐标轴"]["functions"])
    
    # 调用函数
    from link import set_link_local_axes
    set_link_local_axes(model, "1", 30)
"""

# ==================== 枚举类型 ====================
from .enums import (
    LinkType,
    LinkDirectionalType,
    LinkItemType,
    AxisVectorOption,
)

# ==================== 数据类 ====================
from .data_classes import (
    LinkLocalAxesData,
    LinkLocalAxesAdvancedData,
)

# ==================== 局部坐标轴函数 ====================
from .local_axes import (
    set_link_local_axes,
    get_link_local_axes,
    set_link_local_axes_advanced,
    get_link_local_axes_advanced,
    get_link_transformation_matrix,
)

# ==================== 选择函数 ====================
from .selection import (
    set_link_selected,
    get_link_selected,
    select_link,
    deselect_link,
    select_links,
    deselect_links,
    is_link_selected,
)


# ==================== 组分配函数 ====================
from .group import (
    set_link_group,
    add_link_to_group,
    remove_link_from_group,
    get_link_groups,
    is_link_in_group,
    add_links_to_group,
    remove_links_from_group,
)

# ==================== 属性分配函数 ====================
from .property import (
    set_link_property,
    get_link_property,
    set_link_property_fd,
    get_link_property_fd,
    get_link_property_info,
)


# ==================== API 分类索引 (供 AI Agent 发现功能) ====================

LINK_API_CATEGORIES = {
    "局部坐标轴": {
        "description": "设置连接单元的局部坐标轴方向",
        "functions": [
            "set_link_local_axes",           # 设置局部轴角度
            "get_link_local_axes",           # 获取局部轴角度
            "set_link_local_axes_advanced",  # 设置高级局部轴
            "get_link_local_axes_advanced",  # 获取高级局部轴
            "get_link_transformation_matrix", # 获取变换矩阵
        ]
    },
    "选择": {
        "description": "设置连接单元的选择状态",
        "functions": [
            "set_link_selected",   # 设置选择状态
            "get_link_selected",   # 获取选择状态
            "select_link",         # 选中连接单元
            "deselect_link",       # 取消选中连接单元
            "select_links",        # 批量选中连接单元
            "deselect_links",      # 批量取消选中连接单元
            "is_link_selected",    # 检查是否选中
        ]
    },
    "组分配": {
        "description": "设置连接单元的组分配",
        "functions": [
            "set_link_group",           # 设置组分配
            "add_link_to_group",        # 添加到组
            "remove_link_from_group",   # 从组移除
            "get_link_groups",          # 获取所属组
            "is_link_in_group",         # 检查是否在组中
            "add_links_to_group",       # 批量添加到组
            "remove_links_from_group",  # 批量从组移除
        ]
    },
    "属性分配": {
        "description": "设置连接单元的属性分配",
        "functions": [
            "set_link_property",      # 设置连接属性
            "get_link_property",      # 获取连接属性名称
            "set_link_property_fd",   # 设置频率相关属性
            "get_link_property_fd",   # 获取频率相关属性
            "get_link_property_info", # 获取属性信息（含频率相关）
        ]
    },
}


# ==================== 导出列表 ====================

__all__ = [
    # 枚举类型
    'LinkType',
    'LinkDirectionalType',
    'LinkItemType',
    'AxisVectorOption',
    
    # 数据类
    'LinkLocalAxesData',
    'LinkLocalAxesAdvancedData',
    
    # 局部坐标轴函数
    'set_link_local_axes',
    'get_link_local_axes',
    'set_link_local_axes_advanced',
    'get_link_local_axes_advanced',
    'get_link_transformation_matrix',
    
    # 选择函数
    'set_link_selected',
    'get_link_selected',
    'select_link',
    'deselect_link',
    'select_links',
    'deselect_links',
    'is_link_selected',
    
    # 组分配函数
    'set_link_group',
    'add_link_to_group',
    'remove_link_from_group',
    'get_link_groups',
    'is_link_in_group',
    'add_links_to_group',
    'remove_links_from_group',
    
    # 属性分配函数
    'set_link_property',
    'get_link_property',
    'set_link_property_fd',
    'get_link_property_fd',
    'get_link_property_info',
    
    # API 分类索引
    'LINK_API_CATEGORIES',
]



# -*- coding: utf-8 -*-
"""
组操作工具 - 创建组、添加对象到组

重构: 复用 PySap2000.group 和各模块的组分配函数
"""

from langchain.tools import tool

from .base import get_sap_model, success_response, error_response, safe_sap_call

# 导入 PySap2000 封装
from PySap2000.group import Group
from PySap2000.selection import get_selected, SelectObjectType
from PySap2000.frame.group import set_frame_group
from PySap2000.point.constraint import set_point_constraint
from PySap2000.area.group import set_area_group
from PySap2000.cable.group import set_cable_group
from PySap2000.link.group import set_link_group


@tool
@safe_sap_call
def create_group(group_name: str) -> str:
    """
    创建一个新组。
    
    Args:
        group_name: 组名
    """
    model = get_sap_model()
    
    group = Group(name=group_name)
    ret = group._create(model)
    
    if ret == 0:
        return success_response(f"组 '{group_name}' 创建成功")
    return error_response("创建组失败")


@tool
@safe_sap_call
def add_frame_to_group(frame_name: str, group_name: str) -> str:
    """
    将杆件添加到组。
    
    Args:
        frame_name: 杆件名称
        group_name: 组名
    """
    model = get_sap_model()
    
    # 使用 PySap2000 frame.group 模块
    ret = set_frame_group(model, frame_name, group_name)
    
    if ret == 0:
        return success_response(f"杆件 '{frame_name}' 已添加到组 '{group_name}'")
    return error_response("添加失败")


@tool
@safe_sap_call
def add_selected_to_group(group_name: str) -> str:
    """
    将当前选中的对象添加到组。
    
    Args:
        group_name: 组名
    """
    model = get_sap_model()
    
    # 确保组存在
    group = Group(name=group_name)
    group._create(model)
    
    # 获取选中对象
    selected = get_selected(model)
    
    if not selected:
        return error_response("没有选中任何对象")
    
    added = 0
    for obj_type, obj_name in selected:
        try:
            if obj_type == SelectObjectType.POINT.value:
                # Point 使用 structure_core.Point 的方法
                from PySap2000.structure_core import Point
                point = Point(no=obj_name)
                point.set_group_assign(model, group_name)
            elif obj_type == SelectObjectType.FRAME.value:
                set_frame_group(model, obj_name, group_name)
            elif obj_type == SelectObjectType.AREA.value:
                set_area_group(model, obj_name, group_name)
            elif obj_type == SelectObjectType.CABLE.value:
                set_cable_group(model, obj_name, group_name)
            elif obj_type == SelectObjectType.LINK.value:
                set_link_group(model, obj_name, group_name)
            added += 1
        except:
            pass
    
    return success_response(f"已将 {added} 个对象添加到组 '{group_name}'")



# -*- coding: utf-8 -*-
"""
mesh_generator.py - 3D 网格生成器

根据截面轮廓和单元轴线生成 3D 网格数据
用于 Web 查看器的实体模式
"""

import numpy as np
from typing import List, Tuple, Dict
from dataclasses import dataclass
from ..geometry.element_geometry import FrameElement3D, CableElement3D, Point3D
from ..geometry.section_profile import create_profile_from_sap_section


@dataclass
class MeshData:
    """网格数据"""
    vertices: List[float]  # 顶点坐标 [x1,y1,z1, x2,y2,z2, ...]
    normals: List[float]   # 法向量 [nx1,ny1,nz1, ...]
    indices: List[int]     # 三角形索引
    colors: List[float]    # 顶点颜色 [r1,g1,b1, ...]


class MeshGenerator:
    """3D 网格生成器"""
    
    def __init__(self, num_segments: int = 16):
        """
        初始化
        
        Args:
            num_segments: 圆形截面的分段数
        """
        self.num_segments = num_segments
    
    def generate_element_mesh(
        self,
        element,
        color: Tuple[float, float, float] = (0.2, 0.5, 0.9)
    ) -> MeshData:
        """
        为单个单元生成网格
        
        Args:
            element: FrameElement3D 或 CableElement3D
            color: RGB 颜色 (0-1)
            
        Returns:
            MeshData 对象
        """
        # 获取截面轮廓
        if hasattr(element, 'section_type') and element.section_type:
            profile = create_profile_from_sap_section(
                element.section_type,
                element.section_params
            )
            profile.num_segments = self.num_segments
            profile_points = profile.get_profile_points()
        else:
            # 默认圆形截面
            profile_points = self._get_default_circle_profile(0.1)
        
        # 生成拉伸网格
        return self._extrude_profile(
            profile_points,
            element.point_i,
            element.point_j,
            color
        )
    
    def _get_default_circle_profile(self, radius: float) -> List[Tuple[float, float]]:
        """生成默认圆形截面轮廓"""
        points = []
        for i in range(self.num_segments):
            angle = 2 * np.pi * i / self.num_segments
            x = radius * np.cos(angle)
            y = radius * np.sin(angle)
            points.append((x, y))
        return points
    
    def _extrude_profile(
        self,
        profile_points: List[Tuple[float, float]],
        point_i: Point3D,
        point_j: Point3D,
        color: Tuple[float, float, float]
    ) -> MeshData:
        """
        沿轴线拉伸截面轮廓
        
        Args:
            profile_points: 截面轮廓点 [(x1,y1), (x2,y2), ...]
            point_i: 起点
            point_j: 终点
            color: 颜色
            
        Returns:
            MeshData
        """
        # 计算局部坐标系
        axis_x, axis_y, axis_z = self._compute_local_axes(point_i, point_j)
        
        # 转换截面点到 3D 空间
        n_profile = len(profile_points)
        vertices = []
        normals = []
        colors = []
        
        # 起点截面
        for px, py in profile_points:
            # 局部坐标转全局坐标
            point = (
                point_i.x + px * axis_x[0] + py * axis_y[0],
                point_i.y + px * axis_x[1] + py * axis_y[1],
                point_i.z + px * axis_x[2] + py * axis_y[2]
            )
            vertices.extend(point)
            
            # 法向量（径向）
            normal = self._normalize([px * axis_x[i] + py * axis_y[i] for i in range(3)])
            normals.extend(normal)
            
            colors.extend(color)
        
        # 终点截面
        for px, py in profile_points:
            point = (
                point_j.x + px * axis_x[0] + py * axis_y[0],
                point_j.y + px * axis_x[1] + py * axis_y[1],
                point_j.z + px * axis_x[2] + py * axis_y[2]
            )
            vertices.extend(point)
            
            normal = self._normalize([px * axis_x[i] + py * axis_y[i] for i in range(3)])
            normals.extend(normal)
            
            colors.extend(color)
        
        # 生成侧面三角形索引
        indices = []
        for i in range(n_profile):
            i_next = (i + 1) % n_profile
            
            # 第一个三角形
            indices.extend([
                i,                    # 起点当前
                i + n_profile,        # 终点当前
                i_next                # 起点下一个
            ])
            
            # 第二个三角形
            indices.extend([
                i_next,               # 起点下一个
                i + n_profile,        # 终点当前
                i_next + n_profile    # 终点下一个
            ])
        
        # 添加端盖（可选）
        # 起点端盖
        start_cap_indices = self._generate_cap_indices(0, n_profile, False)
        indices.extend(start_cap_indices)
        
        # 终点端盖
        end_cap_indices = self._generate_cap_indices(n_profile, n_profile, True)
        indices.extend(end_cap_indices)
        
        return MeshData(
            vertices=vertices,
            normals=normals,
            indices=indices,
            colors=colors
        )
    
    def _compute_local_axes(
        self,
        point_i: Point3D,
        point_j: Point3D
    ) -> Tuple[List[float], List[float], List[float]]:
        """
        计算局部坐标系
        
        Returns:
            (axis_x, axis_y, axis_z) - 三个轴的方向向量
        """
        # Z 轴：沿单元方向
        axis_z = [
            point_j.x - point_i.x,
            point_j.y - point_i.y,
            point_j.z - point_i.z
        ]
        axis_z = self._normalize(axis_z)
        
        # X 轴：垂直于 Z 轴
        # 选择一个不平行于 Z 轴的向量
        if abs(axis_z[2]) < 0.9:
            up = [0, 0, 1]
        else:
            up = [1, 0, 0]
        
        # X = up × Z
        axis_x = self._cross(up, axis_z)
        axis_x = self._normalize(axis_x)
        
        # Y = Z × X
        axis_y = self._cross(axis_z, axis_x)
        axis_y = self._normalize(axis_y)
        
        return axis_x, axis_y, axis_z
    
    def _generate_cap_indices(
        self,
        start_index: int,
        n_points: int,
        reverse: bool = False
    ) -> List[int]:
        """
        生成端盖的三角形索引（扇形三角化）
        
        Args:
            start_index: 起始顶点索引
            n_points: 轮廓点数
            reverse: 是否反向（法向量方向）
        """
        indices = []
        center_idx = start_index  # 使用第一个点作为中心
        
        for i in range(1, n_points - 1):
            if reverse:
                indices.extend([
                    center_idx,
                    start_index + i + 1,
                    start_index + i
                ])
            else:
                indices.extend([
                    center_idx,
                    start_index + i,
                    start_index + i + 1
                ])
        
        return indices
    
    @staticmethod
    def _normalize(v: List[float]) -> List[float]:
        """归一化向量"""
        length = np.sqrt(sum(x**2 for x in v))
        if length < 1e-10:
            return [0, 0, 1]
        return [x / length for x in v]
    
    @staticmethod
    def _cross(a: List[float], b: List[float]) -> List[float]:
        """向量叉乘"""
        return [
            a[1] * b[2] - a[2] * b[1],
            a[2] * b[0] - a[0] * b[2],
            a[0] * b[1] - a[1] * b[0]
        ]



# -*- coding: utf-8 -*-
"""
analyze - 分析控制模块

SAP2000 的 Analyze API，用于控制分析执行。

SAP2000 API 结构:
- Analyze.RunAnalysis - 运行分析
- Analyze.CreateAnalysisModel - 创建分析模型
- Analyze.DeleteResults - 删除结果
- Analyze.GetActiveDOF / SetActiveDOF - 活动自由度
- Analyze.GetCaseStatus - 工况状态
- Analyze.GetRunCaseFlag / SetRunCaseFlag - 运行标志
- Analyze.GetSolverOption_3 / SetSolverOption_3 - 求解器选项
- Analyze.ModifyUnDeformedGeometry - 修改未变形几何
- Analyze.MergeAnalysisResults - 合并分析结果

Usage:
    from PySap2000.analyze import (
        run_analysis,
        create_analysis_model,
        delete_results,
        ActiveDOF,
        get_active_dof,
        set_active_dof,
        SolverOption,
        get_solver_option,
        set_solver_option,
    )
    
    # 运行分析
    run_analysis(model)
    
    # 设置活动自由度
    dof = ActiveDOF(ux=True, uy=True, uz=True, rx=False, ry=False, rz=False)
    set_active_dof(model, dof)
"""

from .enums import CaseStatus, SolverType, SolverProcessType
from .data_classes import ActiveDOF, SolverOption, CaseStatusInfo, RunCaseFlag
from .analyze import (
    run_analysis,
    create_analysis_model,
    delete_results,
    delete_all_results,
    get_active_dof,
    set_active_dof,
    get_case_status,
    get_run_case_flag,
    set_run_case_flag,
    set_run_case_flag_all,
    get_solver_option,
    set_solver_option,
    modify_undeformed_geometry,
    modify_undeformed_geometry_mode_shape,
    merge_analysis_results,
)

__all__ = [
    # 枚举
    "CaseStatus",
    "SolverType",
    "SolverProcessType",
    # 数据类
    "ActiveDOF",
    "SolverOption",
    "CaseStatusInfo",
    "RunCaseFlag",
    # 核心分析函数
    "run_analysis",
    "create_analysis_model",
    "delete_results",
    "delete_all_results",
    # 自由度
    "get_active_dof",
    "set_active_dof",
    # 工况状态
    "get_case_status",
    "get_run_case_flag",
    "set_run_case_flag",
    "set_run_case_flag_all",
    # 求解器
    "get_solver_option",
    "set_solver_option",
    # 几何修改
    "modify_undeformed_geometry",
    "modify_undeformed_geometry_mode_shape",
    "merge_analysis_results",
]

# AI Agent 友好的 API 分类
ANALYZE_API_CATEGORIES = {
    "core": {
        "description": "核心分析控制",
        "functions": ["run_analysis", "create_analysis_model", "delete_results", "delete_all_results"],
        "api_path": "Analyze",
    },
    "dof": {
        "description": "活动自由度控制",
        "functions": ["get_active_dof", "set_active_dof"],
        "classes": ["ActiveDOF"],
        "api_path": "Analyze.GetActiveDOF/SetActiveDOF",
    },
    "case_control": {
        "description": "工况运行控制",
        "functions": ["get_case_status", "get_run_case_flag", "set_run_case_flag", "set_run_case_flag_all"],
        "classes": ["CaseStatusInfo", "RunCaseFlag"],
        "enums": ["CaseStatus"],
        "api_path": "Analyze.GetCaseStatus/GetRunCaseFlag/SetRunCaseFlag",
    },
    "solver": {
        "description": "求解器选项",
        "functions": ["get_solver_option", "set_solver_option"],
        "classes": ["SolverOption"],
        "enums": ["SolverType", "SolverProcessType"],
        "api_path": "Analyze.GetSolverOption_3/SetSolverOption_3",
    },
    "geometry": {
        "description": "几何修改",
        "functions": ["modify_undeformed_geometry", "modify_undeformed_geometry_mode_shape", "merge_analysis_results"],
        "api_path": "Analyze.ModifyUnDeformedGeometry/MergeAnalysisResults",
    },
}



# -*- coding: utf-8 -*-
"""
modal_results.py - 模态分析结果函数

SAP2000 Results API 的模态分析结果函数封装

SAP2000 API:
- Results.ModalPeriod - 模态周期
- Results.ModeShape - 振型
- Results.ModalParticipatingMassRatios - 模态参与质量比
- Results.ModalLoadParticipationRatios - 模态荷载参与比
- Results.ModalParticipationFactors - 模态参与因子
"""

from typing import List
from .enums import ItemTypeElm
from .data_classes import (
    ModalPeriodResult, ModeShapeResult, ModalMassRatioResult,
    ModalLoadParticipationRatioResult, ModalParticipationFactorResult,
)


def get_modal_period(model) -> List[ModalPeriodResult]:
    """
    获取模态周期结果
    
    返回所有选中模态工况的周期、频率和特征值。
    
    Args:
        model: SapModel 对象
        
    Returns:
        ModalPeriodResult 列表
        
    Example:
        from results import deselect_all_cases_and_combos, set_case_selected_for_output
        
        deselect_all_cases_and_combos(model)
        set_case_selected_for_output(model, "MODAL")
        
        results = get_modal_period(model)
        for r in results:
            print(f"Mode {int(r.step_num)}: T={r.period:.3f}s, f={r.frequency:.3f}Hz")
    """
    result = model.Results.ModalPeriod(
        0, [], [], [],
        [], [], [], []
    )
    
    if isinstance(result, (list, tuple)) and len(result) >= 8:
        num = result[0]
        load_case = result[1]
        step_type = result[2]
        step_num = result[3]
        period = result[4]
        frequency = result[5]
        circ_freq = result[6]
        eigenvalue = result[7]
        ret = result[-1] if len(result) > 8 else 0
        
        if (ret == 0 or len(result) == 8) and num > 0:
            return [
                ModalPeriodResult(
                    load_case=load_case[i] if load_case else "",
                    step_type=step_type[i] if step_type else "",
                    step_num=step_num[i] if step_num else 0.0,
                    period=period[i] if period else 0.0,
                    frequency=frequency[i] if frequency else 0.0,
                    circ_freq=circ_freq[i] if circ_freq else 0.0,
                    eigenvalue=eigenvalue[i] if eigenvalue else 0.0,
                )
                for i in range(num)
            ]
    return []


def get_mode_shape(
    model,
    name: str,
    item_type: ItemTypeElm = ItemTypeElm.GROUP_ELM
) -> List[ModeShapeResult]:
    """
    获取振型结果
    
    Args:
        model: SapModel 对象
        name: 点对象名、点元素名或组名
        item_type: 元素类型
            
    Returns:
        ModeShapeResult 列表
        
    Example:
        from results import deselect_all_cases_and_combos, set_case_selected_for_output
        
        deselect_all_cases_and_combos(model)
        set_case_selected_for_output(model, "MODAL")
        
        # 获取所有点的振型
        results = get_mode_shape(model, "ALL", ItemTypeElm.GROUP_ELM)
    """
    result = model.Results.ModeShape(
        name, int(item_type),
        0, [], [], [], [], [],
        [], [], [], [], [], []
    )
    
    if isinstance(result, (list, tuple)) and len(result) >= 14:
        num = result[0]
        obj = result[1]
        elm = result[2]
        load_case = result[3]
        step_type = result[4]
        step_num = result[5]
        u1 = result[6]
        u2 = result[7]
        u3 = result[8]
        r1 = result[9]
        r2 = result[10]
        r3 = result[11]
        ret = result[-1]
        
        if ret == 0 and num > 0:
            return [
                ModeShapeResult(
                    obj=obj[i] if obj else "",
                    elm=elm[i] if elm else "",
                    load_case=load_case[i] if load_case else "",
                    step_type=step_type[i] if step_type else "",
                    step_num=step_num[i] if step_num else 0.0,
                    u1=u1[i] if u1 else 0.0,
                    u2=u2[i] if u2 else 0.0,
                    u3=u3[i] if u3 else 0.0,
                    r1=r1[i] if r1 else 0.0,
                    r2=r2[i] if r2 else 0.0,
                    r3=r3[i] if r3 else 0.0,
                )
                for i in range(num)
            ]
    return []


def get_modal_participating_mass_ratios(model) -> List[ModalMassRatioResult]:
    """
    获取模态参与质量比
    
    返回各振型的参与质量比和累计参与质量比。
    
    Args:
        model: SapModel 对象
        
    Returns:
        ModalMassRatioResult 列表
        
    Example:
        results = get_modal_participating_mass_ratios(model)
        for r in results:
            print(f"Mode {int(r.step_num)}: "
                  f"Ux={r.ux:.2%}, Uy={r.uy:.2%}, Uz={r.uz:.2%}, "
                  f"SumUx={r.sum_ux:.2%}")
    """
    result = model.Results.ModalParticipatingMassRatios(
        0, [], [], [],
        [],
        [], [], [], [], [], [],
        [], [], [], [], [], []
    )
    
    if isinstance(result, (list, tuple)) and len(result) >= 17:
        num = result[0]
        load_case = result[1]
        step_type = result[2]
        step_num = result[3]
        period = result[4]
        ux = result[5]
        uy = result[6]
        uz = result[7]
        sum_ux = result[8]
        sum_uy = result[9]
        sum_uz = result[10]
        rx = result[11]
        ry = result[12]
        rz = result[13]
        sum_rx = result[14]
        sum_ry = result[15]
        sum_rz = result[16]
        ret = result[-1] if len(result) > 17 else 0
        
        if (ret == 0 or len(result) == 17) and num > 0:
            return [
                ModalMassRatioResult(
                    load_case=load_case[i] if load_case else "",
                    step_type=step_type[i] if step_type else "",
                    step_num=step_num[i] if step_num else 0.0,
                    period=period[i] if period else 0.0,
                    ux=ux[i] if ux else 0.0,
                    uy=uy[i] if uy else 0.0,
                    uz=uz[i] if uz else 0.0,
                    sum_ux=sum_ux[i] if sum_ux else 0.0,
                    sum_uy=sum_uy[i] if sum_uy else 0.0,
                    sum_uz=sum_uz[i] if sum_uz else 0.0,
                    rx=rx[i] if rx else 0.0,
                    ry=ry[i] if ry else 0.0,
                    rz=rz[i] if rz else 0.0,
                    sum_rx=sum_rx[i] if sum_rx else 0.0,
                    sum_ry=sum_ry[i] if sum_ry else 0.0,
                    sum_rz=sum_rz[i] if sum_rz else 0.0,
                )
                for i in range(num)
            ]
    return []


def get_modal_load_participation_ratios(model) -> List[ModalLoadParticipationRatioResult]:
    """
    获取模态荷载参与比
    
    返回各荷载模式的静态和动态参与比。
    
    Args:
        model: SapModel 对象
        
    Returns:
        ModalLoadParticipationRatioResult 列表
    """
    result = model.Results.ModalLoadParticipationRatios(
        0, [], [], [], [], []
    )
    
    if isinstance(result, (list, tuple)) and len(result) >= 6:
        num = result[0]
        load_case = result[1]
        item_type = result[2]
        item = result[3]
        stat = result[4]
        dyn = result[5]
        ret = result[-1] if len(result) > 6 else 0
        
        if (ret == 0 or len(result) == 6) and num > 0:
            return [
                ModalLoadParticipationRatioResult(
                    load_case=load_case[i] if load_case else "",
                    item_type=item_type[i] if item_type else "",
                    item=item[i] if item else "",
                    stat=stat[i] if stat else 0.0,
                    dyn=dyn[i] if dyn else 0.0,
                )
                for i in range(num)
            ]
    return []


def get_modal_participation_factors(model) -> List[ModalParticipationFactorResult]:
    """
    获取模态参与因子
    
    返回各振型的参与因子、模态质量和模态刚度。
    
    Args:
        model: SapModel 对象
        
    Returns:
        ModalParticipationFactorResult 列表
    """
    result = model.Results.ModalParticipationFactors(
        0, [], [], [],
        [],
        [], [], [], [], [], [],
        [], []
    )
    
    if isinstance(result, (list, tuple)) and len(result) >= 13:
        num = result[0]
        load_case = result[1]
        step_type = result[2]
        step_num = result[3]
        period = result[4]
        ux = result[5]
        uy = result[6]
        uz = result[7]
        rx = result[8]
        ry = result[9]
        rz = result[10]
        modal_mass = result[11]
        modal_stiff = result[12]
        ret = result[-1] if len(result) > 13 else 0
        
        if (ret == 0 or len(result) == 13) and num > 0:
            return [
                ModalParticipationFactorResult(
                    load_case=load_case[i] if load_case else "",
                    step_type=step_type[i] if step_type else "",
                    step_num=step_num[i] if step_num else 0.0,
                    period=period[i] if period else 0.0,
                    ux=ux[i] if ux else 0.0,
                    uy=uy[i] if uy else 0.0,
                    uz=uz[i] if uz else 0.0,
                    rx=rx[i] if rx else 0.0,
                    ry=ry[i] if ry else 0.0,
                    rz=rz[i] if rz else 0.0,
                    modal_mass=modal_mass[i] if modal_mass else 0.0,
                    modal_stiff=modal_stiff[i] if modal_stiff else 0.0,
                )
                for i in range(num)
            ]
    return []



# -*- coding: utf-8 -*-
"""
result.py - 统一返回值类型

提供 Result 泛型类，用于统一 API 返回值格式。

Usage:
    from PySap2000.utils.result import Result, Ok, Err
    
    # 成功返回
    def get_point(model, name: str) -> Result[Point]:
        try:
            point = Point.get_by_name(model, name)
            return Ok(point)
        except Exception as e:
            return Err(str(e))
    
    # 使用结果
    result = get_point(model, "1")
    if result.is_ok:
        print(result.data)
    else:
        print(result.error)
"""

from dataclasses import dataclass, field
from typing import Generic, TypeVar, Optional, Union, List, Any

T = TypeVar('T')


@dataclass
class Result(Generic[T]):
    """
    统一的操作结果类型
    
    Attributes:
        success: 操作是否成功
        data: 成功时的返回数据
        error: 失败时的错误信息
        error_code: 错误代码（可选）
        warnings: 警告信息列表
        
    Example:
        result = Result(success=True, data=point)
        result = Result(success=False, error="Point not found")
    """
    success: bool
    data: Optional[T] = None
    error: Optional[str] = None
    error_code: Optional[int] = None
    warnings: List[str] = field(default_factory=list)
    
    @property
    def is_ok(self) -> bool:
        """是否成功"""
        return self.success
    
    @property
    def is_err(self) -> bool:
        """是否失败"""
        return not self.success
    
    def unwrap(self) -> T:
        """
        获取数据，如果失败则抛出异常
        
        Returns:
            成功时的数据
            
        Raises:
            ValueError: 如果操作失败
        """
        if self.success and self.data is not None:
            return self.data
        raise ValueError(self.error or "Operation failed")
    
    def unwrap_or(self, default: T) -> T:
        """
        获取数据，如果失败则返回默认值
        
        Args:
            default: 默认值
            
        Returns:
            成功时的数据或默认值
        """
        if self.success and self.data is not None:
            return self.data
        return default
    
    def map(self, func) -> 'Result':
        """
        对成功的数据应用函数
        
        Args:
            func: 转换函数
            
        Returns:
            新的 Result 对象
        """
        if self.success and self.data is not None:
            try:
                return Result(success=True, data=func(self.data))
            except Exception as e:
                return Result(success=False, error=str(e))
        return Result(success=False, error=self.error)
    
    def __bool__(self) -> bool:
        """允许直接用 if result: 判断"""
        return self.success


def Ok(data: T, warnings: Optional[List[str]] = None) -> Result[T]:
    """
    创建成功结果的快捷函数
    
    Args:
        data: 返回数据
        warnings: 警告信息（可选）
        
    Returns:
        成功的 Result 对象
        
    Example:
        return Ok(point)
        return Ok(frames, warnings=["Some frames were skipped"])
    """
    return Result(success=True, data=data, warnings=warnings or [])


def Err(error: str, error_code: Optional[int] = None) -> Result[Any]:
    """
    创建失败结果的快捷函数
    
    Args:
        error: 错误信息
        error_code: 错误代码（可选）
        
    Returns:
        失败的 Result 对象
        
    Example:
        return Err("Point not found")
        return Err("Connection failed", error_code=-1)
    """
    return Result(success=False, error=error, error_code=error_code)


@dataclass
class BatchResult(Generic[T]):
    """
    批量操作结果类型
    
    Attributes:
        succeeded: 成功的结果列表
        failed: 失败的结果列表（包含错误信息）
        total: 总数
        
    Example:
        result = BatchResult()
        result.add_success(point1)
        result.add_failure("Point2", "Invalid coordinates")
    """
    succeeded: List[T] = field(default_factory=list)
    failed: List[tuple] = field(default_factory=list)  # (item_id, error_message)
    
    @property
    def total(self) -> int:
        """总处理数量"""
        return len(self.succeeded) + len(self.failed)
    
    @property
    def success_count(self) -> int:
        """成功数量"""
        return len(self.succeeded)
    
    @property
    def failure_count(self) -> int:
        """失败数量"""
        return len(self.failed)
    
    @property
    def success_rate(self) -> float:
        """成功率"""
        return self.success_count / self.total if self.total > 0 else 0.0
    
    @property
    def all_succeeded(self) -> bool:
        """是否全部成功"""
        return len(self.failed) == 0
    
    def add_success(self, item: T) -> None:
        """添加成功项"""
        self.succeeded.append(item)
    
    def add_failure(self, item_id: str, error: str) -> None:
        """添加失败项"""
        self.failed.append((item_id, error))
    
    def to_result(self) -> Result[List[T]]:
        """
        转换为 Result 类型
        
        Returns:
            如果全部成功返回 Ok，否则返回包含警告的 Result
        """
        if self.all_succeeded:
            return Ok(self.succeeded)
        else:
            warnings = [f"{item_id}: {error}" for item_id, error in self.failed]
            return Result(
                success=True,
                data=self.succeeded,
                warnings=warnings
            )



# -*- coding: utf-8 -*-
"""
tables.py - 交互式表格编辑核心类

对应 SAP2000 的 DatabaseTables 接口

API Reference:
    - DatabaseTables.GetAvailableTables
    - DatabaseTables.GetAllTables
    - DatabaseTables.GetAllFieldsInTable
    - DatabaseTables.GetTableForDisplayArray
    - DatabaseTables.GetTableForEditingArray
    - DatabaseTables.SetTableForEditingArray
    - DatabaseTables.ApplyEditedTables
    - DatabaseTables.CancelTableEditing
    - DatabaseTables.ShowTablesInExcel
"""

from dataclasses import dataclass, field
from typing import List, Optional, Tuple, Dict, Any, Union
from enum import IntEnum


class TableImportType(IntEnum):
    """表格导入类型"""
    NOT_IMPORTABLE = 0          # 不可导入
    IMPORTABLE = 1              # 可导入
    IMPORTABLE_NOT_INTERACTIVE = 2  # 可导入但不可交互


class TableExportFormat(IntEnum):
    """表格导出格式"""
    ARRAY = 1       # 数组格式
    CSV_FILE = 2    # CSV 文件
    CSV_STRING = 3  # CSV 字符串
    XML_STRING = 4  # XML 字符串


@dataclass
class TableInfo:
    """
    表格信息
    
    Attributes:
        table_key: 表格键名
        table_name: 表格显示名称
        import_type: 导入类型 (0=不可导入, 1=可导入, 2=可导入但不可交互)
        is_empty: 是否为空表格
    """
    table_key: str
    table_name: str = ""
    import_type: int = 0
    is_empty: bool = True
    
    @property
    def is_importable(self) -> bool:
        """是否可导入"""
        return self.import_type in (1, 2)


@dataclass
class TableField:
    """
    表格字段信息
    
    Attributes:
        field_key: 字段键名
        field_name: 字段显示名称
        description: 字段描述
        units: 单位字符串
        is_importable: 是否可导入
    """
    field_key: str
    field_name: str = ""
    description: str = ""
    units: str = ""
    is_importable: bool = True


@dataclass
class TableData:
    """
    表格数据
    
    Attributes:
        table_key: 表格键名
        table_version: 表格版本
        field_keys: 字段键名列表
        num_records: 记录数
        data: 数据列表 (按行展开的一维列表)
    """
    table_key: str = ""
    table_version: int = 0
    field_keys: List[str] = field(default_factory=list)
    num_records: int = 0
    data: List[str] = field(default_factory=list)
    
    @property
    def num_fields(self) -> int:
        """字段数量"""
        return len(self.field_keys)
    
    def get_row(self, row_index: int) -> List[str]:
        """
        获取指定行的数据
        
        Args:
            row_index: 行索引 (0-based)
        
        Returns:
            该行的数据列表
        """
        if row_index < 0 or row_index >= self.num_records:
            return []
        start = row_index * self.num_fields
        end = start + self.num_fields
        return self.data[start:end]
    
    def set_row(self, row_index: int, row_data: List[str]) -> bool:
        """
        设置指定行的数据
        
        Args:
            row_index: 行索引 (0-based)
            row_data: 行数据列表
        
        Returns:
            是否设置成功
        """
        if row_index < 0 or row_index >= self.num_records:
            return False
        if len(row_data) != self.num_fields:
            return False
        start = row_index * self.num_fields
        for i, value in enumerate(row_data):
            self.data[start + i] = str(value)
        return True
    
    def add_row(self, row_data: List[str]) -> bool:
        """
        添加一行数据
        
        Args:
            row_data: 行数据列表
        
        Returns:
            是否添加成功
        """
        if len(row_data) != self.num_fields:
            return False
        self.data.extend([str(v) for v in row_data])
        self.num_records += 1
        return True
    
    def delete_row(self, row_index: int) -> bool:
        """
        删除指定行
        
        Args:
            row_index: 行索引 (0-based)
        
        Returns:
            是否删除成功
        """
        if row_index < 0 or row_index >= self.num_records:
            return False
        start = row_index * self.num_fields
        end = start + self.num_fields
        del self.data[start:end]
        self.num_records -= 1
        return True
    
    def get_value(self, row_index: int, field_name: str) -> Optional[str]:
        """
        获取指定单元格的值
        
        Args:
            row_index: 行索引 (0-based)
            field_name: 字段名称
        
        Returns:
            单元格值，未找到返回 None
        """
        if field_name not in self.field_keys:
            return None
        col_index = self.field_keys.index(field_name)
        data_index = row_index * self.num_fields + col_index
        if 0 <= data_index < len(self.data):
            return self.data[data_index]
        return None
    
    def set_value(self, row_index: int, field_name: str, value: str) -> bool:
        """
        设置指定单元格的值
        
        Args:
            row_index: 行索引 (0-based)
            field_name: 字段名称
            value: 新值
        
        Returns:
            是否设置成功
        """
        if field_name not in self.field_keys:
            return False
        col_index = self.field_keys.index(field_name)
        data_index = row_index * self.num_fields + col_index
        if 0 <= data_index < len(self.data):
            self.data[data_index] = str(value)
            return True
        return False
    
    def get_column(self, field_name: str) -> List[str]:
        """
        获取指定列的所有值
        
        Args:
            field_name: 字段名称
        
        Returns:
            该列的值列表
        """
        if field_name not in self.field_keys:
            return []
        col_index = self.field_keys.index(field_name)
        return [self.data[i * self.num_fields + col_index] 
                for i in range(self.num_records)]
    
    def find_rows(self, field_name: str, value: str) -> List[int]:
        """
        查找字段值匹配的行索引
        
        Args:
            field_name: 字段名称
            value: 要匹配的值
        
        Returns:
            匹配的行索引列表
        """
        result = []
        for i in range(self.num_records):
            if self.get_value(i, field_name) == value:
                result.append(i)
        return result
    
    def to_dict_list(self) -> List[Dict[str, str]]:
        """
        转换为字典列表格式
        
        Returns:
            每行一个字典的列表
        """
        result = []
        for i in range(self.num_records):
            row_data = self.get_row(i)
            row_dict = dict(zip(self.field_keys, row_data))
            result.append(row_dict)
        return result
    
    def from_dict_list(self, dict_list: List[Dict[str, str]]) -> None:
        """
        从字典列表导入数据
        
        Args:
            dict_list: 字典列表
        """
        self.data = []
        self.num_records = 0
        for row_dict in dict_list:
            row_data = [row_dict.get(key, "") for key in self.field_keys]
            self.add_row(row_data)
    
    def to_dataframe(self):
        """
        转换为 pandas DataFrame
        
        Returns:
            pandas.DataFrame 对象
        
        Raises:
            ImportError: 如果未安装 pandas
        """
        try:
            import pandas as pd
            return pd.DataFrame(self.to_dict_list())
        except ImportError:
            raise ImportError("需要安装 pandas: pip install pandas")
    
    def from_dataframe(self, df) -> None:
        """
        从 pandas DataFrame 导入数据
        
        Args:
            df: pandas.DataFrame 对象
        """
        self.from_dict_list(df.to_dict('records'))
    
    def copy(self) -> 'TableData':
        """创建副本"""
        return TableData(
            table_key=self.table_key,
            table_version=self.table_version,
            field_keys=self.field_keys.copy(),
            num_records=self.num_records,
            data=self.data.copy()
        )


@dataclass
class ApplyResult:
    """
    应用编辑结果
    
    Attributes:
        success: 是否成功
        num_fatal_errors: 致命错误数
        num_error_msgs: 错误消息数
        num_warn_msgs: 警告消息数
        num_info_msgs: 信息消息数
        import_log: 导入日志
    """
    success: bool = False
    num_fatal_errors: int = 0
    num_error_msgs: int = 0
    num_warn_msgs: int = 0
    num_info_msgs: int = 0
    import_log: str = ""


class DatabaseTables:
    """
    数据库表格管理类
    
    提供交互式表格编辑的静态方法
    
    典型工作流程:
        1. get_available_tables() - 获取可用表格
        2. get_table_for_editing() - 获取表格数据
        3. 修改 TableData 对象
        4. set_table_for_editing() - 设置修改后的数据
        5. apply_edited_tables() - 应用更改到模型
    
    Example:
        # 读取并修改节点坐标
        data = DatabaseTables.get_table_for_editing(model, "Joint Coordinates")
        data.set_value(0, "XorR", "100")  # 修改第一个节点的 X 坐标
        DatabaseTables.set_table_for_editing(model, data)
        result = DatabaseTables.apply_edited_tables(model)
        if not result.success:
            print(result.import_log)
    """
    
    # ==================== 表格查询 ====================
    
    @staticmethod
    def get_available_tables(model) -> List[TableInfo]:
        """
        获取可用表格列表 (有数据的表格)
        
        Args:
            model: SapModel 对象
        
        Returns:
            TableInfo 对象列表
        
        Example:
            tables = DatabaseTables.get_available_tables(model)
            for t in tables:
                print(f"{t.table_key}: empty={t.is_empty}, importable={t.is_importable}")
        """
        # API: GetAvailableTables(NumberTables, TableKey[], TableName[], ImportType[], IsEmpty[])
        result = model.DatabaseTables.GetAvailableTables(0, [], [], [], [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 5:
            ret = result[-1]
            if ret == 0:
                num_tables = result[0]
                table_keys = list(result[1]) if result[1] else []
                table_names = list(result[2]) if result[2] else []
                import_types = list(result[3]) if result[3] else []
                is_empty = list(result[4]) if result[4] else []
                
                tables = []
                for i in range(num_tables):
                    tables.append(TableInfo(
                        table_key=table_keys[i] if i < len(table_keys) else "",
                        table_name=table_names[i] if i < len(table_names) else "",
                        import_type=import_types[i] if i < len(import_types) else 0,
                        is_empty=is_empty[i] if i < len(is_empty) else True
                    ))
                return tables
        return []
    
    @staticmethod
    def get_available_table_keys(model) -> List[str]:
        """
        获取可用表格键名列表 (简化版)
        
        Args:
            model: SapModel 对象
        
        Returns:
            表格键名列表
        """
        tables = DatabaseTables.get_available_tables(model)
        return [t.table_key for t in tables]
    
    @staticmethod
    def get_all_tables(model) -> List[TableInfo]:
        """
        获取所有表格列表 (包括空表格)
        
        Args:
            model: SapModel 对象
        
        Returns:
            TableInfo 对象列表
        """
        # API: GetAllTables(NumberTables, TableKey[], TableName[], ImportType[])
        result = model.DatabaseTables.GetAllTables(0, [], [], [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 4:
            ret = result[-1]
            if ret == 0:
                num_tables = result[0]
                table_keys = list(result[1]) if result[1] else []
                table_names = list(result[2]) if result[2] else []
                import_types = list(result[3]) if result[3] else []
                
                tables = []
                for i in range(num_tables):
                    tables.append(TableInfo(
                        table_key=table_keys[i] if i < len(table_keys) else "",
                        table_name=table_names[i] if i < len(table_names) else "",
                        import_type=import_types[i] if i < len(import_types) else 0,
                        is_empty=True  # GetAllTables 不返回 IsEmpty
                    ))
                return tables
        return []
    
    @staticmethod
    def get_all_table_keys(model) -> List[str]:
        """
        获取所有表格键名列表 (简化版)
        
        Args:
            model: SapModel 对象
        
        Returns:
            表格键名列表
        """
        tables = DatabaseTables.get_all_tables(model)
        return [t.table_key for t in tables]
    
    @staticmethod
    def get_fields_in_table(model, table_key: str, table_version: int = 0) -> List[TableField]:
        """
        获取表格中的所有字段
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            table_version: 表格版本 (默认 0)
        
        Returns:
            TableField 对象列表
        
        Example:
            fields = DatabaseTables.get_fields_in_table(model, "Joint Coordinates")
            for f in fields:
                print(f"{f.field_key}: {f.description} ({f.units})")
        """
        # API: GetAllFieldsInTable(TableKey, TableVersion, NumberFields, 
        #      FieldKey[], FieldName[], Description[], UnitsString[], IsImportable[])
        # 返回: [TableVersion, NumberFields, FieldKey[], FieldName[], Description[], UnitsString[], IsImportable[], ret]
        result = model.DatabaseTables.GetAllFieldsInTable(
            table_key, table_version, 0, [], [], [], [], []
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 8:
            ret = result[-1]
            if ret == 0:
                num_fields = result[1]
                field_keys = list(result[2]) if result[2] else []
                field_names = list(result[3]) if result[3] else []
                descriptions = list(result[4]) if result[4] else []
                units = list(result[5]) if result[5] else []
                is_importable = list(result[6]) if result[6] else []
                
                fields = []
                for i in range(num_fields):
                    fields.append(TableField(
                        field_key=field_keys[i] if i < len(field_keys) else "",
                        field_name=field_names[i] if i < len(field_names) else "",
                        description=descriptions[i] if i < len(descriptions) else "",
                        units=units[i] if i < len(units) else "",
                        is_importable=is_importable[i] if i < len(is_importable) else True
                    ))
                return fields
        return []
    
    @staticmethod
    def get_all_fields_in_table(
        model,
        table_key: str,
        table_version: int = 0
    ) -> Tuple[List[str], List[str], List[str], List[str], List[bool]]:
        """
        获取表格中的所有字段 (原始格式)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            table_version: 表格版本
        
        Returns:
            (field_keys, field_names, descriptions, units, is_importable) 元组
        """
        fields = DatabaseTables.get_fields_in_table(model, table_key, table_version)
        return (
            [f.field_key for f in fields],
            [f.field_name for f in fields],
            [f.description for f in fields],
            [f.units for f in fields],
            [f.is_importable for f in fields]
        )

    
    # ==================== 读取表格 ====================
    
    @staticmethod
    def get_table_for_display(
        model,
        table_key: str,
        field_keys: List[str] = None,
        group_name: str = ""
    ) -> Optional[TableData]:
        """
        获取显示用表格数据 (Array 格式)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            field_keys: 要获取的字段列表，None 或 [""] 表示所有字段
            group_name: 组名称，空字符串或 "All" 表示所有对象
        
        Returns:
            TableData 对象，失败返回 None
        
        Example:
            data = DatabaseTables.get_table_for_display(
                model, 
                "Frame Section Assignments"
            )
            if data:
                for row in data.to_dict_list():
                    print(row)
        """
        # 如果 field_keys 为 None 或空，使用 [""] 表示所有字段
        if field_keys is None or len(field_keys) == 0:
            field_keys = [""]
        
        # API: GetTableForDisplayArray(TableKey, FieldKeyList[], GroupName,
        #      TableVersion, FieldKeysIncluded[], NumberRecords, TableData[])
        # 返回格式: [输入的field_keys, TableVersion, FieldKeysIncluded, NumberRecords, TableData, ret]
        result = model.DatabaseTables.GetTableForDisplayArray(
            table_key,
            field_keys,
            group_name,
            0,      # TableVersion (output)
            [],     # FieldKeysIncluded (output)
            0,      # NumberRecords (output)
            []      # TableData (output)
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 6:
            ret = result[-1]  # 返回码在最后
            if ret == 0:
                return TableData(
                    table_key=table_key,
                    table_version=result[1],  # TableVersion
                    field_keys=list(result[2]) if result[2] else [],  # FieldKeysIncluded
                    num_records=result[3],  # NumberRecords
                    data=list(result[4]) if result[4] else []  # TableData
                )
        return None
    
    @staticmethod
    def get_table_for_editing(
        model,
        table_key: str,
        group_name: str = ""
    ) -> Optional[TableData]:
        """
        获取编辑用表格数据 (Array 格式)
        
        与 get_table_for_display 类似，但返回可编辑的数据。
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            group_name: 组名称 (当前版本未激活此参数)
        
        Returns:
            TableData 对象，失败返回 None
        
        Example:
            data = DatabaseTables.get_table_for_editing(model, "Joint Coordinates")
            data.set_value(0, "XorR", "100")
            DatabaseTables.set_table_for_editing(model, data)
            DatabaseTables.apply_edited_tables(model)
        """
        # API: GetTableForEditingArray(TableKey, GroupName,
        #      TableVersion, FieldKeysIncluded[], NumberRecords, TableData[])
        # 返回: [TableVersion, FieldKeysIncluded[], NumberRecords, TableData[], ret]
        result = model.DatabaseTables.GetTableForEditingArray(
            table_key,
            group_name,
            0,      # TableVersion (output)
            [],     # FieldKeysIncluded (output)
            0,      # NumberRecords (output)
            []      # TableData (output)
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 5:
            ret = result[-1]
            if ret == 0:
                return TableData(
                    table_key=table_key,
                    table_version=result[0],
                    field_keys=list(result[1]) if result[1] else [],
                    num_records=result[2],
                    data=list(result[3]) if result[3] else []
                )
        return None

    
    # ==================== 编辑表格 ====================
    
    @staticmethod
    def set_table_for_editing(
        model,
        table_data: TableData
    ) -> int:
        """
        设置编辑表格数据 (推荐方式)
        
        Args:
            model: SapModel 对象
            table_data: TableData 对象
        
        Returns:
            0 表示成功，非零表示失败
        
        Example:
            data = DatabaseTables.get_table_for_editing(model, "Joint Coordinates")
            data.set_value(0, "XorR", "100")
            ret = DatabaseTables.set_table_for_editing(model, data)
            if ret == 0:
                DatabaseTables.apply_edited_tables(model)
        """
        # API: SetTableForEditingArray(TableKey, TableVersion, 
        #      FieldKeysIncluded[], NumberRecords, TableData[])
        result = model.DatabaseTables.SetTableForEditingArray(
            table_data.table_key,
            table_data.table_version,
            table_data.field_keys,
            table_data.num_records,
            table_data.data
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def set_table_for_editing_array(
        model,
        table_key: str,
        table_version: int,
        field_keys: List[str],
        num_records: int,
        data: List[str]
    ) -> int:
        """
        设置编辑表格数据 (原始参数方式)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            table_version: 表格版本
            field_keys: 字段键名列表
            num_records: 记录数
            data: 数据列表
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetTableForEditingArray(
            table_key,
            table_version,
            field_keys,
            num_records,
            data
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def apply_edited_tables(
        model,
        fill_import_log: bool = True
    ) -> ApplyResult:
        """
        应用已编辑的表格
        
        将所有通过 set_table_for_editing 设置的更改应用到模型。
        
        Args:
            model: SapModel 对象
            fill_import_log: 是否填充导入日志
        
        Returns:
            ApplyResult 对象
        
        Example:
            DatabaseTables.set_table_for_editing(model, data)
            result = DatabaseTables.apply_edited_tables(model)
            if not result.success:
                print(f"错误: {result.num_fatal_errors} 个致命错误")
                print(result.import_log)
        """
        # API: ApplyEditedTables(FillImportLog, NumFatalErrors, NumErrorMsgs, 
        #      NumWarnMsgs, NumInfoMsgs, ImportLog)
        result = model.DatabaseTables.ApplyEditedTables(
            fill_import_log, 0, 0, 0, 0, ""
        )
        
        apply_result = ApplyResult()
        
        if isinstance(result, (list, tuple)) and len(result) >= 6:
            ret = result[-1]
            apply_result.success = (ret == 0)
            apply_result.num_fatal_errors = result[1] if len(result) > 1 else 0
            apply_result.num_error_msgs = result[2] if len(result) > 2 else 0
            apply_result.num_warn_msgs = result[3] if len(result) > 3 else 0
            apply_result.num_info_msgs = result[4] if len(result) > 4 else 0
            apply_result.import_log = result[5] if len(result) > 5 else ""
        
        return apply_result
    
    @staticmethod
    def cancel_table_editing(model) -> int:
        """
        取消表格编辑
        
        取消所有未应用的表格编辑。
        
        Args:
            model: SapModel 对象
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.CancelTableEditing()
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    # ==================== 显示选项 ====================
    
    @staticmethod
    def set_load_patterns_selected(
        model,
        load_patterns: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的荷载模式
        
        Args:
            model: SapModel 对象
            load_patterns: 荷载模式名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetLoadPatternsSelectedForDisplay(
            len(load_patterns),
            load_patterns,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def set_load_cases_selected(
        model,
        load_cases: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的荷载工况
        
        Args:
            model: SapModel 对象
            load_cases: 荷载工况名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetLoadCasesSelectedForDisplay(
            len(load_cases),
            load_cases,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def set_load_combinations_selected(
        model,
        load_combos: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的荷载组合
        
        Args:
            model: SapModel 对象
            load_combos: 荷载组合名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetLoadCombinationsSelectedForDisplay(
            len(load_combos),
            load_combos,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    # ==================== 导出 ====================
    
    @staticmethod
    def show_tables_in_excel(
        model,
        table_keys: List[str],
        field_keys_list: List[List[str]] = None,
        group_name: str = ""
    ) -> int:
        """
        在 Excel 中显示表格
        
        Args:
            model: SapModel 对象
            table_keys: 表格键名列表
            field_keys_list: 每个表格的字段键名列表 (可选)
            group_name: 组名称 (可选)
        
        Returns:
            0 表示成功
        
        Example:
            DatabaseTables.show_tables_in_excel(
                model,
                ["Frame Section Assignments", "Joint Coordinates"]
            )
        """
        num_tables = len(table_keys)
        
        if field_keys_list is None:
            field_keys_list = [[] for _ in table_keys]
        
        # 每个表格的字段数
        num_fields_per_table = [len(fields) for fields in field_keys_list]
        
        # 展平字段列表
        all_field_keys = []
        for fields in field_keys_list:
            all_field_keys.extend(fields)
        
        result = model.DatabaseTables.ShowTablesInExcel(
            num_tables,
            table_keys,
            num_fields_per_table,
            all_field_keys,
            group_name
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    # ==================== 便捷方法 ====================
    
    @staticmethod
    def read_table(
        model,
        table_key: str,
        as_dataframe: bool = False
    ):
        """
        读取表格数据 (便捷方法)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            as_dataframe: 是否返回 pandas DataFrame
        
        Returns:
            TableData 对象或 pandas DataFrame
        
        Example:
            # 返回 TableData
            data = DatabaseTables.read_table(model, "Joint Coordinates")
            
            # 返回 DataFrame
            df = DatabaseTables.read_table(model, "Joint Coordinates", as_dataframe=True)
        """
        data = DatabaseTables.get_table_for_display(model, table_key)
        if data and as_dataframe:
            return data.to_dataframe()
        return data
    
    @staticmethod
    def edit_table(
        model,
        table_key: str,
        modifications: Dict[int, Dict[str, str]]
    ) -> ApplyResult:
        """
        编辑表格数据 (便捷方法)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            modifications: 修改字典 {行索引: {字段名: 新值}}
        
        Returns:
            ApplyResult 对象
        
        Example:
            # 修改第0行的 X 坐标和第1行的 Y 坐标
            result = DatabaseTables.edit_table(model, "Joint Coordinates", {
                0: {"XorR": "100"},
                1: {"Y": "200"}
            })
        """
        data = DatabaseTables.get_table_for_editing(model, table_key)
        if not data:
            return ApplyResult(success=False, import_log="无法获取表格数据")
        
        for row_idx, field_values in modifications.items():
            for field_name, value in field_values.items():
                data.set_value(row_idx, field_name, value)
        
        ret = DatabaseTables.set_table_for_editing(model, data)
        if ret != 0:
            return ApplyResult(success=False, import_log="设置表格数据失败")
        
        return DatabaseTables.apply_edited_tables(model)
    
    @staticmethod
    def import_from_dataframe(
        model,
        table_key: str,
        df
    ) -> ApplyResult:
        """
        从 pandas DataFrame 导入数据
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            df: pandas DataFrame
        
        Returns:
            ApplyResult 对象
        
        Example:
            import pandas as pd
            df = pd.DataFrame({
                'Joint': ['1', '2'],
                'XorR': ['0', '100'],
                'Y': ['0', '0'],
                'Z': ['0', '0']
            })
            result = DatabaseTables.import_from_dataframe(model, "Joint Coordinates", df)
        """
        data = DatabaseTables.get_table_for_editing(model, table_key)
        if not data:
            return ApplyResult(success=False, import_log="无法获取表格数据")
        
        data.from_dataframe(df)
        
        ret = DatabaseTables.set_table_for_editing(model, data)
        if ret != 0:
            return ApplyResult(success=False, import_log="设置表格数据失败")
        
        return DatabaseTables.apply_edited_tables(model)

    # ==================== CSV 格式方法 ====================
    
    @staticmethod
    def get_table_for_display_csv_file(
        model,
        table_key: str,
        file_path: str,
        field_keys: List[str] = None,
        group_name: str = ""
    ) -> int:
        """
        获取显示用表格数据并保存为 CSV 文件
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            file_path: CSV 文件路径
            field_keys: 要获取的字段列表，None 表示所有字段
            group_name: 组名称，空字符串表示所有对象
        
        Returns:
            0 表示成功
        """
        if field_keys is None:
            field_keys = []
        
        result = model.DatabaseTables.GetTableForDisplayCSVFile(
            table_key,
            field_keys,
            group_name,
            file_path
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def get_table_for_display_csv_string(
        model,
        table_key: str,
        field_keys: List[str] = None,
        group_name: str = ""
    ) -> Tuple[str, int]:
        """
        获取显示用表格数据为 CSV 字符串
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            field_keys: 要获取的字段列表，None 表示所有字段
            group_name: 组名称，空字符串表示所有对象
        
        Returns:
            (csv_string, ret) 元组
        """
        if field_keys is None:
            field_keys = []
        
        result = model.DatabaseTables.GetTableForDisplayCSVString(
            table_key,
            field_keys,
            group_name,
            ""  # CSVString (output)
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            return result[-2], result[-1]
        return "", -1
    
    @staticmethod
    def get_table_for_editing_csv_file(
        model,
        table_key: str,
        file_path: str,
        field_keys: List[str] = None
    ) -> int:
        """
        获取编辑用表格数据并保存为 CSV 文件
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            file_path: CSV 文件路径
            field_keys: 要获取的字段列表，None 表示所有字段
        
        Returns:
            0 表示成功
        """
        if field_keys is None:
            field_keys = []
        
        result = model.DatabaseTables.GetTableForEditingCSVFile(
            table_key,
            field_keys,
            file_path
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def get_table_for_editing_csv_string(
        model,
        table_key: str,
        field_keys: List[str] = None
    ) -> Tuple[str, int]:
        """
        获取编辑用表格数据为 CSV 字符串
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            field_keys: 要获取的字段列表，None 表示所有字段
        
        Returns:
            (csv_string, ret) 元组
        """
        if field_keys is None:
            field_keys = []
        
        result = model.DatabaseTables.GetTableForEditingCSVString(
            table_key,
            field_keys,
            ""  # CSVString (output)
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            return result[-2], result[-1]
        return "", -1
    
    @staticmethod
    def set_table_for_editing_csv_file(
        model,
        table_key: str,
        file_path: str
    ) -> int:
        """
        从 CSV 文件设置编辑表格数据
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            file_path: CSV 文件路径
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetTableForEditingCSVFile(
            table_key,
            file_path
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def set_table_for_editing_csv_string(
        model,
        table_key: str,
        csv_string: str
    ) -> int:
        """
        从 CSV 字符串设置编辑表格数据
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            csv_string: CSV 格式字符串
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetTableForEditingCSVString(
            table_key,
            csv_string
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    # ==================== 获取显示选项 ====================
    
    @staticmethod
    def get_load_patterns_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的荷载模式
        
        Args:
            model: SapModel 对象
        
        Returns:
            (load_patterns, ret) 元组
        """
        result = model.DatabaseTables.GetLoadPatternsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def get_load_cases_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的荷载工况
        
        Args:
            model: SapModel 对象
        
        Returns:
            (load_cases, ret) 元组
        """
        result = model.DatabaseTables.GetLoadCasesSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def get_load_combinations_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的荷载组合
        
        Args:
            model: SapModel 对象
        
        Returns:
            (load_combos, ret) 元组
        """
        result = model.DatabaseTables.GetLoadCombinationsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    # ==================== Named Sets 方法 ====================
    
    @staticmethod
    def get_section_cuts_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的截面切割
        
        Args:
            model: SapModel 对象
        
        Returns:
            (section_cuts, ret) 元组
        """
        result = model.DatabaseTables.GetSectionCutsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def set_section_cuts_selected(
        model,
        section_cuts: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的截面切割
        
        Args:
            model: SapModel 对象
            section_cuts: 截面切割名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetSectionCutsSelectedForDisplay(
            len(section_cuts),
            section_cuts,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def get_generalized_displacements_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的广义位移
        
        Args:
            model: SapModel 对象
        
        Returns:
            (generalized_displacements, ret) 元组
        """
        result = model.DatabaseTables.GetGeneralizedDisplacementsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def set_generalized_displacements_selected(
        model,
        generalized_displacements: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的广义位移
        
        Args:
            model: SapModel 对象
            generalized_displacements: 广义位移名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetGeneralizedDisplacementsSelectedForDisplay(
            len(generalized_displacements),
            generalized_displacements,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1

    
    @staticmethod
    def get_pushover_named_sets_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的 Pushover 命名集
        
        Args:
            model: SapModel 对象
        
        Returns:
            (named_sets, ret) 元组
        """
        result = model.DatabaseTables.GetPushoverNamedSetsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def set_pushover_named_sets_selected(
        model,
        named_sets: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的 Pushover 命名集
        
        Args:
            model: SapModel 对象
            named_sets: 命名集名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetPushoverNamedSetsSelectedForDisplay(
            len(named_sets),
            named_sets,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def get_joint_response_spectra_named_sets_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的节点反应谱命名集
        
        Args:
            model: SapModel 对象
        
        Returns:
            (named_sets, ret) 元组
        """
        result = model.DatabaseTables.GetJointResponseSpectraNamedSetsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def set_joint_response_spectra_named_sets_selected(
        model,
        named_sets: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的节点反应谱命名集
        
        Args:
            model: SapModel 对象
            named_sets: 命名集名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetJointResponseSpectraNamedSetsSelectedForDisplay(
            len(named_sets),
            named_sets,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def get_plot_function_traces_named_sets_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的绘图函数轨迹命名集
        
        Args:
            model: SapModel 对象
        
        Returns:
            (named_sets, ret) 元组
        """
        result = model.DatabaseTables.GetPlotFunctionTracesNamedSetsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def set_plot_function_traces_named_sets_selected(
        model,
        named_sets: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的绘图函数轨迹命名集
        
        Args:
            model: SapModel 对象
            named_sets: 命名集名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetPlotFunctionTracesNamedSetsSelectedForDisplay(
            len(named_sets),
            named_sets,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    @staticmethod
    def get_element_virtual_work_named_sets_selected(model) -> Tuple[List[str], int]:
        """
        获取当前选中显示的单元虚功命名集
        
        Args:
            model: SapModel 对象
        
        Returns:
            (named_sets, ret) 元组
        """
        result = model.DatabaseTables.GetElementVirtualWorkNamedSetsSelectedForDisplay(0, [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            ret = result[-1]
            if ret == 0:
                return list(result[1]) if result[1] else [], ret
        return [], -1
    
    @staticmethod
    def set_element_virtual_work_named_sets_selected(
        model,
        named_sets: List[str],
        selected: bool = True
    ) -> int:
        """
        设置显示的单元虚功命名集
        
        Args:
            model: SapModel 对象
            named_sets: 命名集名称列表
            selected: True=选中, False=取消选中
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetElementVirtualWorkNamedSetsSelectedForDisplay(
            len(named_sets),
            named_sets,
            selected
        )
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    # ==================== 输出选项 ====================
    
    @staticmethod
    def get_table_output_options(model) -> Tuple[Dict[str, Any], int]:
        """
        获取表格输出选项
        
        Args:
            model: SapModel 对象
        
        Returns:
            (options_dict, ret) 元组
            options_dict 包含:
                - joints: 节点输出选项 (0=All, 1=Selected, 2=None)
                - frames: 杆件输出选项
                - cables: 索输出选项
                - tendons: 预应力筋输出选项
                - areas: 面单元输出选项
                - solids: 实体单元输出选项
                - links: 连接单元输出选项
        """
        # API: GetTableOutputOptionsForDisplay(Joints, Frames, Cables, Tendons, Areas, Solids, Links)
        result = model.DatabaseTables.GetTableOutputOptionsForDisplay(
            0, 0, 0, 0, 0, 0, 0
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 8:
            ret = result[-1]
            if ret == 0:
                return {
                    "joints": result[0],
                    "frames": result[1],
                    "cables": result[2],
                    "tendons": result[3],
                    "areas": result[4],
                    "solids": result[5],
                    "links": result[6]
                }, ret
        return {}, -1
    
    @staticmethod
    def set_table_output_options(
        model,
        joints: int = 0,
        frames: int = 0,
        cables: int = 0,
        tendons: int = 0,
        areas: int = 0,
        solids: int = 0,
        links: int = 0
    ) -> int:
        """
        设置表格输出选项
        
        Args:
            model: SapModel 对象
            joints: 节点输出选项 (0=All, 1=Selected, 2=None)
            frames: 杆件输出选项
            cables: 索输出选项
            tendons: 预应力筋输出选项
            areas: 面单元输出选项
            solids: 实体单元输出选项
            links: 连接单元输出选项
        
        Returns:
            0 表示成功
        """
        result = model.DatabaseTables.SetTableOutputOptionsForDisplay(
            joints, frames, cables, tendons, areas, solids, links
        )
        
        if isinstance(result, (list, tuple)):
            return result[-1] if result else -1
        return result if isinstance(result, int) else -1
    
    # ==================== 工具方法 ====================
    
    @staticmethod
    def get_obsolete_table_keys(model) -> Tuple[Dict[str, str], int]:
        """
        获取废弃表格键名映射
        
        返回旧表格键名到新表格键名的映射。
        
        Args:
            model: SapModel 对象
        
        Returns:
            (mapping_dict, ret) 元组
            mapping_dict: {旧键名: 新键名}
        """
        # API: GetObsoleteTableKeyList(NumberItems, OldKey[], NewKey[])
        result = model.DatabaseTables.GetObsoleteTableKeyList(0, [], [])
        
        if isinstance(result, (list, tuple)) and len(result) >= 4:
            ret = result[-1]
            if ret == 0:
                num_items = result[0]
                old_keys = list(result[1]) if result[1] else []
                new_keys = list(result[2]) if result[2] else []
                
                mapping = {}
                for i in range(num_items):
                    if i < len(old_keys) and i < len(new_keys):
                        mapping[old_keys[i]] = new_keys[i]
                return mapping, ret
        return {}, -1
    
    @staticmethod
    def find_tables(model, keyword: str, include_empty: bool = False) -> List[TableInfo]:
        """
        搜索表格 (便捷方法)
        
        根据关键字搜索表格名称。
        
        Args:
            model: SapModel 对象
            keyword: 搜索关键字 (不区分大小写)
            include_empty: 是否包含空表格
        
        Returns:
            匹配的 TableInfo 列表
        
        Example:
            # 搜索所有包含 "Frame" 的表格
            tables = DatabaseTables.find_tables(model, "Frame")
            for t in tables:
                print(t.table_key)
        """
        if include_empty:
            all_tables = DatabaseTables.get_all_tables(model)
        else:
            all_tables = DatabaseTables.get_available_tables(model)
        
        keyword_lower = keyword.lower()
        return [t for t in all_tables 
                if keyword_lower in t.table_key.lower() or keyword_lower in t.table_name.lower()]
    
    @staticmethod
    def export_to_csv(
        model,
        table_key: str,
        file_path: str,
        for_editing: bool = False
    ) -> int:
        """
        导出表格到 CSV 文件 (便捷方法)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            file_path: CSV 文件路径
            for_editing: True=导出可编辑格式, False=导出显示格式
        
        Returns:
            0 表示成功
        
        Example:
            DatabaseTables.export_to_csv(model, "Joint Coordinates", "joints.csv")
        """
        if for_editing:
            return DatabaseTables.get_table_for_editing_csv_file(model, table_key, file_path)
        else:
            return DatabaseTables.get_table_for_display_csv_file(model, table_key, file_path)
    
    @staticmethod
    def import_from_csv(
        model,
        table_key: str,
        file_path: str,
        apply_immediately: bool = True
    ) -> ApplyResult:
        """
        从 CSV 文件导入表格数据 (便捷方法)
        
        Args:
            model: SapModel 对象
            table_key: 表格键名
            file_path: CSV 文件路径
            apply_immediately: 是否立即应用更改
        
        Returns:
            ApplyResult 对象
        
        Example:
            result = DatabaseTables.import_from_csv(model, "Joint Coordinates", "joints.csv")
            if not result.success:
                print(result.import_log)
        """
        ret = DatabaseTables.set_table_for_editing_csv_file(model, table_key, file_path)
        if ret != 0:
            return ApplyResult(success=False, import_log="设置 CSV 文件失败")
        
        if apply_immediately:
            return DatabaseTables.apply_edited_tables(model)
        else:
            return ApplyResult(success=True, import_log="数据已加载，请调用 apply_edited_tables 应用更改")



# -*- coding: utf-8 -*-
"""
SAP2000 RAG 知识库模块

基于 CSI_AI_Ready_Texts 文档实现检索增强生成（RAG）功能。
支持中文分词、关键词匹配、TF-IDF 加权。

特性：
- 中文分词支持（jieba，可选）
- 搜索结果缓存
- 文档结构化解析（函数签名、参数、返回值）
- 多种搜索策略
"""

import os
import re
import hashlib
from pathlib import Path
from typing import List, Tuple, Optional, Dict, Set
from dataclasses import dataclass, field
from functools import lru_cache
import threading
import time

# 尝试导入 jieba，不强制依赖
try:
    import jieba
    JIEBA_AVAILABLE = True
except ImportError:
    JIEBA_AVAILABLE = False


@dataclass
class FunctionInfo:
    """函数信息（从文档中提取）"""
    name: str = ""
    signature: str = ""
    description: str = ""
    parameters: List[str] = field(default_factory=list)
    returns: str = ""
    remarks: str = ""


@dataclass
class Document:
    """文档数据类"""
    filename: str
    content: str
    title: str  # 从文件名提取的标题
    # 结构化信息
    function_info: Optional[FunctionInfo] = None
    # 分词后的词集合（用于快速匹配）
    tokens: Set[str] = field(default_factory=set)
    # 关键词（高权重词）
    keywords: Set[str] = field(default_factory=set)


class SAP2000KnowledgeBase:
    """SAP2000 知识库"""
    
    # 搜索缓存配置
    CACHE_SIZE = 100  # 缓存最近 100 次搜索
    CACHE_TTL = 3600  # 缓存有效期 1 小时
    
    def __init__(self, docs_dir: Optional[str] = None):
        """
        初始化知识库
        
        Args:
            docs_dir: 文档目录路径，优先级：
                      1. 传入参数
                      2. 环境变量 SAP_DOCS_PATH
                      3. 默认相对路径 docs/CSI_AI_Ready_Texts
        """
        if docs_dir is None:
            docs_dir = os.getenv("SAP_DOCS_PATH")
        
        if docs_dir is None:
            current_dir = Path(__file__).parent.parent
            docs_dir = current_dir / "docs" / "CSI_AI_Ready_Texts"
        
        self.docs_dir = Path(docs_dir)
        self.documents: List[Document] = []
        
        # 搜索缓存
        self._cache: Dict[str, Tuple[List[Tuple[Document, float]], float]] = {}
        self._cache_lock = threading.Lock()
        
        # 倒排索引（词 -> 文档索引列表）
        self._inverted_index: Dict[str, Set[int]] = {}
        
        self._load_documents()
        self._build_index()
    
    def _load_documents(self):
        """加载所有文档"""
        if not self.docs_dir.exists():
            print(f"[RAG] 文档目录不存在: {self.docs_dir}")
            return
        
        for filepath in self.docs_dir.glob("*.txt"):
            try:
                content = filepath.read_text(encoding='utf-8', errors='ignore')
                title = self._extract_title(filepath.stem)
                
                # 解析函数信息
                func_info = self._parse_function_info(content)
                
                # 分词
                tokens = self._tokenize(content + " " + title)
                
                # 提取关键词
                keywords = self._extract_keywords(filepath.stem, func_info)
                
                self.documents.append(Document(
                    filename=filepath.name,
                    content=content,
                    title=title,
                    function_info=func_info,
                    tokens=tokens,
                    keywords=keywords
                ))
            except Exception as e:
                print(f"[RAG] 加载文档失败 {filepath}: {e}")
        
        print(f"[RAG] 已加载 {len(self.documents)} 个文档")
        if JIEBA_AVAILABLE:
            print("[RAG] 中文分词已启用 (jieba)")
    
    def _build_index(self):
        """构建倒排索引"""
        for idx, doc in enumerate(self.documents):
            for token in doc.tokens:
                if token not in self._inverted_index:
                    self._inverted_index[token] = set()
                self._inverted_index[token].add(idx)
            # 关键词也加入索引
            for kw in doc.keywords:
                if kw not in self._inverted_index:
                    self._inverted_index[kw] = set()
                self._inverted_index[kw].add(idx)
        
        print(f"[RAG] 倒排索引已构建，共 {len(self._inverted_index)} 个词条")
    
    def _extract_title(self, filename: str) -> str:
        """从文件名提取可读标题"""
        title = filename
        prefixes = [
            "SAP2000_API_Fuctions_",  # 注意原文档有拼写错误
            "SAP2000_API_Functions_",
            "Database_Tables_",
        ]
        for prefix in prefixes:
            if title.startswith(prefix):
                title = title[len(prefix):]
                break
        
        # 替换下划线为空格
        title = title.replace("_", " ")
        title = re.sub(r'\s+', ' ', title).strip()
        return title
    
    def _parse_function_info(self, content: str) -> Optional[FunctionInfo]:
        """从文档内容解析函数信息"""
        info = FunctionInfo()
        
        # 提取函数名（通常在开头）
        name_match = re.search(r'^([A-Za-z_][A-Za-z0-9_\.]+)\s*[\(\n]', content, re.MULTILINE)
        if name_match:
            info.name = name_match.group(1)
        
        # 提取函数签名
        sig_match = re.search(r'((?:Function|Sub|def)\s+[^\n]+)', content, re.IGNORECASE)
        if sig_match:
            info.signature = sig_match.group(1).strip()
        
        # 提取描述（通常在 Description 或开头几行）
        desc_match = re.search(r'Description[:\s]*\n([^\n]+(?:\n[^\n]+)*?)(?=\n\s*\n|\nParameters|\nRemarks|\nReturn)', content, re.IGNORECASE)
        if desc_match:
            info.description = desc_match.group(1).strip()
        
        # 提取参数
        params_match = re.search(r'Parameters[:\s]*\n((?:[^\n]+\n)*?)(?=\n\s*\n|\nRemarks|\nReturn|\nExample)', content, re.IGNORECASE)
        if params_match:
            params_text = params_match.group(1)
            # 解析每个参数
            param_lines = [line.strip() for line in params_text.split('\n') if line.strip()]
            info.parameters = param_lines[:10]  # 最多保留 10 个参数
        
        # 提取返回值
        return_match = re.search(r'Return[s]?[:\s]*\n([^\n]+)', content, re.IGNORECASE)
        if return_match:
            info.returns = return_match.group(1).strip()
        
        # 提取备注
        remarks_match = re.search(r'Remarks[:\s]*\n([^\n]+(?:\n[^\n]+)*?)(?=\n\s*\n|\nExample|\nSee Also|$)', content, re.IGNORECASE)
        if remarks_match:
            info.remarks = remarks_match.group(1).strip()[:500]  # 限制长度
        
        return info if info.name or info.description else None
    
    def _tokenize(self, text: str) -> Set[str]:
        """
        分词
        
        支持中英文混合分词：
        - 英文：按空格和标点分割
        - 中文：使用 jieba 分词（如果可用）
        """
        tokens = set()
        text_lower = text.lower()
        
        # 英文分词
        english_tokens = re.findall(r'[a-zA-Z_][a-zA-Z0-9_]*', text_lower)
        tokens.update(t for t in english_tokens if len(t) >= 2)
        
        # 中文分词
        if JIEBA_AVAILABLE:
            chinese_text = re.sub(r'[a-zA-Z0-9_]+', ' ', text)
            chinese_tokens = jieba.cut(chinese_text)
            tokens.update(t.strip() for t in chinese_tokens if len(t.strip()) >= 2)
        else:
            # 降级：简单按标点分割中文
            chinese_parts = re.findall(r'[\u4e00-\u9fff]+', text)
            for part in chinese_parts:
                # 2-4 字的中文词
                for i in range(len(part)):
                    for length in [2, 3, 4]:
                        if i + length <= len(part):
                            tokens.add(part[i:i+length])
        
        return tokens
    
    def _extract_keywords(self, filename: str, func_info: Optional[FunctionInfo]) -> Set[str]:
        """提取关键词（高权重词）"""
        keywords = set()
        
        # 从文件名提取
        # 例如: "Object_Model_Frame_Object_AddByCoord" -> ["frame", "addbycoord", "object"]
        parts = filename.lower().split('_')
        keywords.update(p for p in parts if len(p) >= 3 and p not in {'sap2000', 'api', 'fuctions', 'functions', 'the', 'and', 'for'})
        
        # 从函数信息提取
        if func_info:
            if func_info.name:
                # 分割驼峰命名
                name_parts = re.findall(r'[A-Z]?[a-z]+|[A-Z]+(?=[A-Z]|$)', func_info.name)
                keywords.update(p.lower() for p in name_parts if len(p) >= 3)
                keywords.add(func_info.name.lower())
        
        return keywords
    
    def _calculate_relevance(self, query: str, doc: Document, query_tokens: Set[str]) -> float:
        """
        计算查询与文档的相关性分数
        
        使用多种信号：
        1. 关键词匹配（高权重）
        2. 标题匹配
        3. 内容词频
        4. 完整查询匹配
        """
        score = 0.0
        query_lower = query.lower()
        
        # 1. 关键词匹配（权重最高）
        keyword_matches = query_tokens & doc.keywords
        score += len(keyword_matches) * 8.0
        
        # 2. 标题匹配
        title_lower = doc.title.lower()
        for token in query_tokens:
            if token in title_lower:
                score += 5.0
        
        # 3. 内容词匹配
        token_matches = query_tokens & doc.tokens
        score += len(token_matches) * 2.0
        
        # 4. 完整查询匹配
        content_lower = doc.content.lower()
        if query_lower in content_lower:
            score += 15.0
        elif query_lower in title_lower:
            score += 20.0
        
        # 5. 函数名精确匹配
        if doc.function_info and doc.function_info.name:
            func_name_lower = doc.function_info.name.lower()
            if query_lower in func_name_lower or func_name_lower in query_lower:
                score += 25.0
        
        return score
    
    def _get_cache_key(self, query: str, top_k: int) -> str:
        """生成缓存键"""
        return hashlib.md5(f"{query}:{top_k}".encode()).hexdigest()
    
    def _get_from_cache(self, cache_key: str) -> Optional[List[Tuple[Document, float]]]:
        """从缓存获取结果"""
        with self._cache_lock:
            if cache_key in self._cache:
                results, timestamp = self._cache[cache_key]
                if time.time() - timestamp < self.CACHE_TTL:
                    return results
                else:
                    del self._cache[cache_key]
        return None
    
    def _set_cache(self, cache_key: str, results: List[Tuple[Document, float]]):
        """设置缓存"""
        with self._cache_lock:
            # 清理过期缓存
            if len(self._cache) >= self.CACHE_SIZE:
                current_time = time.time()
                expired_keys = [
                    k for k, (_, ts) in self._cache.items()
                    if current_time - ts > self.CACHE_TTL
                ]
                for k in expired_keys:
                    del self._cache[k]
                # 如果还是满了，删除最旧的
                if len(self._cache) >= self.CACHE_SIZE:
                    oldest_key = min(self._cache.keys(), key=lambda k: self._cache[k][1])
                    del self._cache[oldest_key]
            
            self._cache[cache_key] = (results, time.time())
    
    def search(self, query: str, top_k: int = 3) -> List[Tuple[Document, float]]:
        """
        搜索相关文档
        
        Args:
            query: 查询字符串
            top_k: 返回前 k 个结果
            
        Returns:
            (文档, 相关性分数) 列表
        """
        if not self.documents:
            return []
        
        # 检查缓存
        cache_key = self._get_cache_key(query, top_k)
        cached = self._get_from_cache(cache_key)
        if cached is not None:
            return cached
        
        # 分词
        query_tokens = self._tokenize(query)
        
        # 使用倒排索引快速筛选候选文档
        candidate_indices: Set[int] = set()
        for token in query_tokens:
            if token in self._inverted_index:
                candidate_indices.update(self._inverted_index[token])
        
        # 如果没有匹配，搜索所有文档
        if not candidate_indices:
            candidate_indices = set(range(len(self.documents)))
        
        # 计算相关性
        results = []
        for idx in candidate_indices:
            doc = self.documents[idx]
            score = self._calculate_relevance(query, doc, query_tokens)
            if score > 0:
                results.append((doc, score))
        
        # 排序
        results.sort(key=lambda x: x[1], reverse=True)
        results = results[:top_k]
        
        # 缓存结果
        self._set_cache(cache_key, results)
        
        return results
    
    def get_context(self, query: str, max_chars: int = 4000) -> str:
        """
        获取查询相关的上下文文本
        
        Args:
            query: 查询字符串
            max_chars: 最大字符数
            
        Returns:
            相关文档内容的拼接
        """
        results = self.search(query, top_k=3)
        
        if not results:
            return ""
        
        context_parts = []
        total_chars = 0
        
        for doc, score in results:
            content = doc.content[:1500]
            
            if total_chars + len(content) > max_chars:
                remaining = max_chars - total_chars
                if remaining > 200:
                    content = content[:remaining]
                else:
                    break
            
            context_parts.append(f"### {doc.title}\n{content}")
            total_chars += len(content)
        
        return "\n\n".join(context_parts)
    
    def clear_cache(self):
        """清空搜索缓存"""
        with self._cache_lock:
            self._cache.clear()


# 全局知识库实例（懒加载）
_knowledge_base: Optional[SAP2000KnowledgeBase] = None
_kb_lock = threading.Lock()


def get_knowledge_base() -> SAP2000KnowledgeBase:
    """获取知识库单例（线程安全）"""
    global _knowledge_base
    if _knowledge_base is None:
        with _kb_lock:
            if _knowledge_base is None:
                _knowledge_base = SAP2000KnowledgeBase()
    return _knowledge_base


def search_sap_docs(query: str, top_k: int = 3) -> str:
    """
    搜索 SAP2000 文档（供 LangChain 工具使用）
    
    Args:
        query: 搜索查询
        top_k: 返回结果数量
        
    Returns:
        搜索结果的格式化字符串
    """
    kb = get_knowledge_base()
    results = kb.search(query, top_k)
    
    if not results:
        return "未找到相关文档。建议：\n1. 尝试使用英文关键词（如 FrameObj, AddByPoint）\n2. 使用更具体的 API 名称"
    
    output = []
    for i, (doc, score) in enumerate(results, 1):
        # 构建结果
        parts = [f"**{i}. {doc.title}** (相关度: {score:.1f})"]
        
        # 添加函数信息（如果有）
        if doc.function_info:
            fi = doc.function_info
            if fi.name:
                parts.append(f"   函数: `{fi.name}`")
            if fi.signature:
                parts.append(f"   签名: `{fi.signature[:100]}`")
            if fi.description:
                parts.append(f"   描述: {fi.description[:200]}")
            if fi.parameters:
                params_str = "; ".join(fi.parameters[:5])
                if len(fi.parameters) > 5:
                    params_str += f" ... (共{len(fi.parameters)}个参数)"
                parts.append(f"   参数: {params_str[:200]}")
            if fi.returns:
                parts.append(f"   返回: {fi.returns[:100]}")
        else:
            # 没有结构化信息，显示内容摘要
            summary = doc.content[:300].replace('\n', ' ')
            parts.append(f"   摘要: {summary}...")
        
        output.append("\n".join(parts))
    
    return "\n\n".join(output)



# -*- coding: utf-8 -*-
"""
link_load.py - 连接单元荷载

包含:
- 枚举: LinkLoadItemType
- 数据类: LinkLoadDeformationData, LinkLoadGravityData, LinkLoadTargetForceData
- 函数: set_link_load_xxx, get_link_load_xxx, delete_link_load_xxx

SAP2000 API:
- LinkObj.SetLoadDeformation / GetLoadDeformation / DeleteLoadDeformation
- LinkObj.SetLoadGravity / GetLoadGravity / DeleteLoadGravity
- LinkObj.SetLoadTargetForce / GetLoadTargetForce / DeleteLoadTargetForce
"""

from dataclasses import dataclass, field
from typing import List, Tuple
from enum import IntEnum


# ==================== 枚举 ====================

class LinkLoadItemType(IntEnum):
    """荷载应用对象类型"""
    OBJECT = 0              # 单个对象
    GROUP = 1               # 组
    SELECTED_OBJECTS = 2    # 选中对象


# ==================== 数据类 ====================

@dataclass
class LinkLoadDeformationData:
    """连接单元变形荷载数据"""
    link_name: str = ""
    load_pattern: str = ""
    dof: Tuple[bool, ...] = field(default_factory=lambda: (False,) * 6)  # U1,U2,U3,R1,R2,R3
    deformation: Tuple[float, ...] = field(default_factory=lambda: (0.0,) * 6)  # [L] or [rad]


@dataclass
class LinkLoadGravityData:
    """连接单元重力荷载数据"""
    link_name: str = ""
    load_pattern: str = ""
    x: float = 0.0
    y: float = 0.0
    z: float = -1.0
    csys: str = "Global"


@dataclass
class LinkLoadTargetForceData:
    """连接单元目标力荷载数据"""
    link_name: str = ""
    load_pattern: str = ""
    dof: Tuple[bool, ...] = field(default_factory=lambda: (False,) * 6)  # P,V2,V3,T,M2,M3
    force: Tuple[float, ...] = field(default_factory=lambda: (0.0,) * 6)  # [F] or [FL]
    relative_dist: Tuple[float, ...] = field(default_factory=lambda: (0.5,) * 6)


# ==================== 变形荷载函数 ====================

def set_link_load_deformation(
    model,
    link_name: str,
    load_pattern: str,
    dof: Tuple[bool, ...],
    deformation: Tuple[float, ...],
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> int:
    """
    设置连接单元变形荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        load_pattern: 荷载模式名称
        dof: 各自由度是否有变形荷载 (U1, U2, U3, R1, R2, R3)
        deformation: 变形值 (U1, U2, U3 [L], R1, R2, R3 [rad])
        item_type: 操作范围
    
    Returns:
        0 表示成功
    
    Example:
        set_link_load_deformation(model, "1", "DEAD", 
            (True, False, False, False, False, False), (0.01, 0, 0, 0, 0, 0))
    """
    dof_list = list(dof) if len(dof) >= 6 else list(dof) + [False] * (6 - len(dof))
    d_list = list(deformation) if len(deformation) >= 6 else list(deformation) + [0.0] * (6 - len(deformation))
    
    return model.LinkObj.SetLoadDeformation(
        str(link_name), load_pattern, dof_list, d_list, int(item_type)
    )


def get_link_load_deformation(
    model,
    link_name: str,
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> List[LinkLoadDeformationData]:
    """
    获取连接单元变形荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        item_type: 操作范围
    
    Returns:
        LinkLoadDeformationData 对象列表
    """
    loads = []
    try:
        result = model.LinkObj.GetLoadDeformation(str(link_name), int(item_type))
        if isinstance(result, (list, tuple)) and len(result) >= 15:
            num_items = result[0]
            if num_items > 0:
                link_names = result[1]
                load_pats = result[2]
                dof1 = result[3]
                dof2 = result[4]
                dof3 = result[5]
                dof4 = result[6]
                dof5 = result[7]
                dof6 = result[8]
                u1 = result[9]
                u2 = result[10]
                u3 = result[11]
                r1 = result[12]
                r2 = result[13]
                r3 = result[14]
                
                for i in range(num_items):
                    loads.append(LinkLoadDeformationData(
                        link_name=link_names[i] if link_names else str(link_name),
                        load_pattern=load_pats[i] if load_pats else "",
                        dof=(dof1[i], dof2[i], dof3[i], dof4[i], dof5[i], dof6[i]),
                        deformation=(u1[i], u2[i], u3[i], r1[i], r2[i], r3[i])
                    ))
    except Exception:
        pass
    return loads


def delete_link_load_deformation(
    model,
    link_name: str,
    load_pattern: str,
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> int:
    """
    删除连接单元变形荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        load_pattern: 荷载模式名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    """
    return model.LinkObj.DeleteLoadDeformation(str(link_name), load_pattern, int(item_type))


# ==================== 重力荷载函数 ====================

def set_link_load_gravity(
    model,
    link_name: str,
    load_pattern: str,
    x: float = 0.0,
    y: float = 0.0,
    z: float = -1.0,
    replace: bool = True,
    csys: str = "Global",
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> int:
    """
    设置连接单元重力荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        load_pattern: 荷载模式名称
        x: X方向重力系数
        y: Y方向重力系数
        z: Z方向重力系数 (默认-1)
        replace: True=替换现有荷载, False=叠加
        csys: 坐标系名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    """
    return model.LinkObj.SetLoadGravity(
        str(link_name), load_pattern, x, y, z, replace, csys, int(item_type)
    )


def get_link_load_gravity(
    model,
    link_name: str,
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> List[LinkLoadGravityData]:
    """
    获取连接单元重力荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        item_type: 操作范围
    
    Returns:
        LinkLoadGravityData 对象列表
    """
    loads = []
    try:
        result = model.LinkObj.GetLoadGravity(str(link_name), int(item_type))
        if isinstance(result, (list, tuple)) and len(result) >= 7:
            num_items = result[0]
            if num_items > 0:
                link_names = result[1]
                load_pats = result[2]
                csys_list = result[3]
                x_list = result[4]
                y_list = result[5]
                z_list = result[6]
                
                for i in range(num_items):
                    loads.append(LinkLoadGravityData(
                        link_name=link_names[i] if link_names else str(link_name),
                        load_pattern=load_pats[i] if load_pats else "",
                        x=x_list[i] if x_list else 0.0,
                        y=y_list[i] if y_list else 0.0,
                        z=z_list[i] if z_list else 0.0,
                        csys=csys_list[i] if csys_list else "Global"
                    ))
    except Exception:
        pass
    return loads


def delete_link_load_gravity(
    model,
    link_name: str,
    load_pattern: str,
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> int:
    """
    删除连接单元重力荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        load_pattern: 荷载模式名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    """
    return model.LinkObj.DeleteLoadGravity(str(link_name), load_pattern, int(item_type))


# ==================== 目标力荷载函数 ====================

def set_link_load_target_force(
    model,
    link_name: str,
    load_pattern: str,
    dof: Tuple[bool, ...],
    force: Tuple[float, ...],
    relative_dist: Tuple[float, ...],
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> int:
    """
    设置连接单元目标力荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        load_pattern: 荷载模式名称
        dof: 各自由度是否有目标力 (P, V2, V3, T, M2, M3)
        force: 目标力值 (P [F], V2 [F], V3 [F], T [FL], M2 [FL], M3 [FL])
        relative_dist: 相对距离 (0-1)
        item_type: 操作范围
    
    Returns:
        0 表示成功
    """
    dof_list = list(dof) if len(dof) >= 6 else list(dof) + [False] * (6 - len(dof))
    f_list = list(force) if len(force) >= 6 else list(force) + [0.0] * (6 - len(force))
    rd_list = list(relative_dist) if len(relative_dist) >= 6 else list(relative_dist) + [0.5] * (6 - len(relative_dist))
    
    return model.LinkObj.SetLoadTargetForce(
        str(link_name), load_pattern, dof_list, f_list, rd_list, int(item_type)
    )


def get_link_load_target_force(
    model,
    link_name: str,
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> List[LinkLoadTargetForceData]:
    """
    获取连接单元目标力荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        item_type: 操作范围
    
    Returns:
        LinkLoadTargetForceData 对象列表
    """
    loads = []
    try:
        result = model.LinkObj.GetLoadTargetForce(str(link_name), int(item_type))
        if isinstance(result, (list, tuple)) and len(result) >= 21:
            num_items = result[0]
            if num_items > 0:
                link_names = result[1]
                load_pats = result[2]
                dof1 = result[3]
                dof2 = result[4]
                dof3 = result[5]
                dof4 = result[6]
                dof5 = result[7]
                dof6 = result[8]
                p_vals = result[9]
                v2_vals = result[10]
                v3_vals = result[11]
                t_vals = result[12]
                m2_vals = result[13]
                m3_vals = result[14]
                t1_vals = result[15]
                t2_vals = result[16]
                t3_vals = result[17]
                t4_vals = result[18]
                t5_vals = result[19]
                t6_vals = result[20]
                
                for i in range(num_items):
                    loads.append(LinkLoadTargetForceData(
                        link_name=link_names[i] if link_names else str(link_name),
                        load_pattern=load_pats[i] if load_pats else "",
                        dof=(dof1[i], dof2[i], dof3[i], dof4[i], dof5[i], dof6[i]),
                        force=(p_vals[i], v2_vals[i], v3_vals[i], t_vals[i], m2_vals[i], m3_vals[i]),
                        relative_dist=(t1_vals[i], t2_vals[i], t3_vals[i], t4_vals[i], t5_vals[i], t6_vals[i])
                    ))
    except Exception:
        pass
    return loads


def delete_link_load_target_force(
    model,
    link_name: str,
    load_pattern: str,
    item_type: LinkLoadItemType = LinkLoadItemType.OBJECT
) -> int:
    """
    删除连接单元目标力荷载
    
    Args:
        model: SapModel 对象
        link_name: 连接单元名称
        load_pattern: 荷载模式名称
        item_type: 操作范围
    
    Returns:
        0 表示成功
    """
    return model.LinkObj.DeleteLoadTargetForce(str(link_name), load_pattern, int(item_type))



# -*- coding: utf-8 -*-
"""
load_case.py - 荷载工况定义

对应 SAP2000 的 LoadCases API

荷载工况（Load Case）是分析工况，定义了如何分析结构。
每种工况类型有不同的子类型和参数设置。

SAP2000 API 结构:
- LoadCases (基础 API)
  - ChangeName, Count, Delete, GetNameList_1, GetTypeOAPI_2, SetDesignType
- LoadCases.StaticLinear (线性静力)
- LoadCases.StaticNonlinear (非线性静力)
- LoadCases.ModalEigen (特征值模态)
- LoadCases.ModalRitz (Ritz 模态)
- LoadCases.ResponseSpectrum (反应谱)
- LoadCases.DirHistLinear (直接积分线性时程)
- LoadCases.DirHistNonlinear (直接积分非线性时程)
- LoadCases.ModHistLinear (模态线性时程)
- LoadCases.ModHistNonlinear (模态非线性时程)
- LoadCases.Buckling (屈曲)
- LoadCases.SteadyState (稳态)
- LoadCases.PSD (功率谱密度)
- LoadCases.MovingLoad (移动荷载)
- LoadCases.Hyperstatic (超静定)
- LoadCases.StaticLinearMultistep (多步线性静力)
- LoadCases.StaticNonlinearMultistep (多步非线性静力)
- LoadCases.StaticNonlinearStaged (分阶段施工)

Usage:
    from PySap2000.loading import LoadCase, LoadCaseType
    
    # 获取所有荷载工况
    all_cases = LoadCase.get_all(model)
    
    # 按类型获取
    static_cases = LoadCase.get_name_list(model, LoadCaseType.LINEAR_STATIC)
    
    # 获取工况信息
    case = LoadCase.get_by_name(model, "DEAD")
    print(f"Type: {case.case_type.name}")
"""

from dataclasses import dataclass, field
from typing import List, Optional, ClassVar, Union, Tuple
from enum import IntEnum

from .load_pattern import LoadPatternType


class LoadCaseType(IntEnum):
    """
    荷载工况类型
    
    对应 SAP2000 的 eLoadCaseType 枚举
    """
    LINEAR_STATIC = 1               # 线性静力
    NONLINEAR_STATIC = 2            # 非线性静力
    MODAL = 3                       # 模态
    RESPONSE_SPECTRUM = 4           # 反应谱
    LINEAR_HISTORY = 5              # 模态线性时程
    NONLINEAR_HISTORY = 6           # 模态非线性时程
    LINEAR_DYNAMIC = 7              # 直接积分线性时程
    NONLINEAR_DYNAMIC = 8           # 直接积分非线性时程
    MOVING_LOAD = 9                 # 移动荷载
    BUCKLING = 10                   # 屈曲
    STEADY_STATE = 11               # 稳态
    POWER_SPECTRAL_DENSITY = 12     # 功率谱密度
    LINEAR_STATIC_MULTISTEP = 13    # 多步线性静力
    HYPERSTATIC = 14                # 超静定
    EXTERNAL_RESULTS = 15           # 外部结果
    STAGED_CONSTRUCTION = 16        # 分阶段施工
    NONLINEAR_STATIC_MULTISTEP = 17 # 多步非线性静力


class ModalSubType(IntEnum):
    """
    模态工况子类型
    
    仅适用于 LoadCaseType.MODAL
    """
    EIGEN = 1   # 特征值模态
    RITZ = 2    # Ritz 模态


class TimeHistorySubType(IntEnum):
    """
    时程工况子类型
    
    仅适用于 LoadCaseType.LINEAR_HISTORY
    """
    TRANSIENT = 1   # 瞬态
    PERIODIC = 2    # 周期


class DesignTypeOption(IntEnum):
    """
    设计类型选项
    """
    PROGRAM_DETERMINED = 0  # 程序自动确定
    USER_SPECIFIED = 1      # 用户指定


@dataclass
class LoadCaseLoad:
    """
    荷载工况中的单个荷载定义
    
    用于 StaticLinear 等工况的荷载设置
    
    Attributes:
        load_type: "Load" (荷载模式) 或 "Accel" (加速度)
        load_name: 荷载模式名称 或 方向 (UX, UY, UZ, RX, RY, RZ)
        scale_factor: 比例系数
    """
    load_type: str = "Load"     # "Load" or "Accel"
    load_name: str = ""         # Pattern name or direction
    scale_factor: float = 1.0


@dataclass
class LoadCase:
    """
    荷载工况定义
    
    对应 SAP2000 的 LoadCases
    
    这是一个基础类，提供所有工况类型的通用操作。
    具体工况类型的详细设置需要通过对应的子 API 进行。
    
    Attributes:
        name: 工况名称
        case_type: 工况类型 (LoadCaseType)
        sub_type: 子类型 (仅 Modal 和 LinearHistory 有效)
        design_type: 设计类型 (LoadPatternType)
        design_type_option: 设计类型选项
        is_auto: 是否自动创建
    """
    name: str = ""
    case_type: LoadCaseType = LoadCaseType.LINEAR_STATIC
    sub_type: int = 0
    design_type: LoadPatternType = LoadPatternType.DEAD
    design_type_option: DesignTypeOption = DesignTypeOption.PROGRAM_DETERMINED
    is_auto: bool = False
    
    _object_type: ClassVar[str] = "LoadCases"

    
    def _get(self, model) -> int:
        """
        从模型获取荷载工况数据
        
        Args:
            model: SapModel 对象
            
        Returns:
            0 表示成功
        """
        result = model.LoadCases.GetTypeOAPI_2(
            self.name, 0, 0, 0, 0, 0
        )
        
        if isinstance(result, (list, tuple)) and len(result) >= 6:
            try:
                self.case_type = LoadCaseType(result[0])
            except ValueError:
                self.case_type = LoadCaseType.LINEAR_STATIC
            
            self.sub_type = result[1]
            
            try:
                self.design_type = LoadPatternType(result[2])
            except ValueError:
                self.design_type = LoadPatternType.OTHER
            
            try:
                self.design_type_option = DesignTypeOption(result[3])
            except ValueError:
                self.design_type_option = DesignTypeOption.PROGRAM_DETERMINED
            
            self.is_auto = bool(result[4])
            ret = result[5]
        else:
            ret = -1
        
        return ret
    
    def _delete(self, model) -> int:
        """
        删除荷载工况
        
        Args:
            model: SapModel 对象
            
        Returns:
            0 表示成功
        """
        return model.LoadCases.Delete(self.name)
    
    def change_name(self, model, new_name: str) -> int:
        """
        重命名荷载工况
        
        Args:
            model: SapModel 对象
            new_name: 新名称
            
        Returns:
            0 表示成功
        """
        ret = model.LoadCases.ChangeName(self.name, new_name)
        if ret == 0:
            self.name = new_name
        return ret
    
    def set_design_type(
        self, 
        model, 
        design_type_option: DesignTypeOption,
        design_type: LoadPatternType = LoadPatternType.DEAD
    ) -> int:
        """
        设置设计类型
        
        Args:
            model: SapModel 对象
            design_type_option: 设计类型选项 (程序确定/用户指定)
            design_type: 设计类型 (仅当 design_type_option=USER_SPECIFIED 时有效)
            
        Returns:
            0 表示成功
        """
        ret = model.LoadCases.SetDesignType(
            self.name,
            int(design_type_option),
            int(design_type)
        )
        if ret == 0:
            self.design_type_option = design_type_option
            if design_type_option == DesignTypeOption.USER_SPECIFIED:
                self.design_type = design_type
        return ret
    
    @staticmethod
    def get_count(model, case_type: Optional[LoadCaseType] = None) -> int:
        """
        获取荷载工况数量
        
        Args:
            model: SapModel 对象
            case_type: 工况类型 (可选，不指定则返回所有类型的总数)
            
        Returns:
            荷载工况数量
        """
        if case_type is None:
            return model.LoadCases.Count()
        else:
            return model.LoadCases.Count(int(case_type))
    
    @staticmethod
    def get_name_list(
        model, 
        case_type: Optional[LoadCaseType] = None
    ) -> List[str]:
        """
        获取荷载工况名称列表
        
        Args:
            model: SapModel 对象
            case_type: 工况类型 (可选，不指定则返回所有类型)
            
        Returns:
            荷载工况名称列表
        """
        if case_type is None:
            result = model.LoadCases.GetNameList_1(0, [])
        else:
            result = model.LoadCases.GetNameList_1(0, [], int(case_type))
        
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            names = result[1]
            if names:
                return list(names)
        
        return []
    
    @classmethod
    def get_by_name(cls, model, name: str) -> Optional["LoadCase"]:
        """
        按名称获取荷载工况
        
        Args:
            model: SapModel 对象
            name: 工况名称
            
        Returns:
            LoadCase 对象，如果不存在返回 None
        """
        case = cls(name=name)
        ret = case._get(model)
        if ret == 0:
            return case
        return None
    
    @classmethod
    def get_all(
        cls, 
        model, 
        case_type: Optional[LoadCaseType] = None
    ) -> List["LoadCase"]:
        """
        获取所有荷载工况
        
        Args:
            model: SapModel 对象
            case_type: 工况类型 (可选，不指定则返回所有类型)
            
        Returns:
            LoadCase 对象列表
        """
        names = cls.get_name_list(model, case_type)
        result = []
        for name in names:
            case = cls.get_by_name(model, name)
            if case:
                result.append(case)
        return result
    
    def get_modal_sub_type(self) -> Optional[ModalSubType]:
        """
        获取模态子类型
        
        仅当 case_type == MODAL 时有效
        
        Returns:
            ModalSubType 或 None
        """
        if self.case_type == LoadCaseType.MODAL and self.sub_type in (1, 2):
            return ModalSubType(self.sub_type)
        return None
    
    def get_time_history_sub_type(self) -> Optional[TimeHistorySubType]:
        """
        获取时程子类型
        
        仅当 case_type == LINEAR_HISTORY 时有效
        
        Returns:
            TimeHistorySubType 或 None
        """
        if self.case_type == LoadCaseType.LINEAR_HISTORY and self.sub_type in (1, 2):
            return TimeHistorySubType(self.sub_type)
        return None


# =============================================================================
# 静力工况创建函数
# =============================================================================

def create_static_linear_case(model, name: str) -> int:
    """
    创建线性静力工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.StaticLinear.SetCase(name)


def create_static_nonlinear_case(model, name: str) -> int:
    """
    创建非线性静力工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.StaticNonlinear.SetCase(name)


# =============================================================================
# 模态工况创建函数
# =============================================================================

def create_modal_eigen_case(model, name: str) -> int:
    """
    创建特征值模态工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.ModalEigen.SetCase(name)


def create_modal_ritz_case(model, name: str) -> int:
    """
    创建 Ritz 模态工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.ModalRitz.SetCase(name)


# =============================================================================
# 动力工况创建函数
# =============================================================================

def create_response_spectrum_case(model, name: str) -> int:
    """
    创建反应谱工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.ResponseSpectrum.SetCase(name)


def create_buckling_case(model, name: str) -> int:
    """
    创建屈曲工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.Buckling.SetCase(name)


# =============================================================================
# 时程工况创建函数
# =============================================================================

def create_direct_history_linear_case(model, name: str) -> int:
    """
    创建直接积分线性时程工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.DirHistLinear.SetCase(name)


def create_direct_history_nonlinear_case(model, name: str) -> int:
    """
    创建直接积分非线性时程工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.DirHistNonlinear.SetCase(name)


def create_modal_history_linear_case(model, name: str) -> int:
    """
    创建模态线性时程工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.ModHistLinear.SetCase(name)


def create_modal_history_nonlinear_case(model, name: str) -> int:
    """
    创建模态非线性时程工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.ModHistNonlinear.SetCase(name)


# =============================================================================
# 其他工况创建函数
# =============================================================================

def create_steady_state_case(model, name: str) -> int:
    """
    创建稳态工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.SteadyState.SetCase(name)


def create_psd_case(model, name: str) -> int:
    """
    创建功率谱密度工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.PSD.SetCase(name)


def create_moving_load_case(model, name: str) -> int:
    """
    创建移动荷载工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.MovingLoad.SetCase(name)


def create_hyperstatic_case(model, name: str) -> int:
    """
    创建超静定工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.Hyperstatic.SetCase(name)


# =============================================================================
# 多步工况创建函数
# =============================================================================

def create_static_linear_multistep_case(model, name: str) -> int:
    """
    创建多步线性静力工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.StaticLinearMultistep.SetCase(name)


def create_static_nonlinear_multistep_case(model, name: str) -> int:
    """
    创建多步非线性静力工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.StaticNonlinearMultistep.SetCase(name)


def create_staged_construction_case(model, name: str) -> int:
    """
    创建分阶段施工工况
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        0 表示成功
    """
    return model.LoadCases.StaticNonlinearStaged.SetCase(name)


# =============================================================================
# 静力工况荷载设置函数
# =============================================================================

def get_static_linear_loads(
    model, 
    name: str
) -> Tuple[List[LoadCaseLoad], int]:
    """
    获取线性静力工况的荷载数据
    
    Args:
        model: SapModel 对象
        name: 工况名称
        
    Returns:
        (荷载列表, 返回码)
    """
    result = model.LoadCases.StaticLinear.GetLoads(name, 0, [], [], [])
    
    loads = []
    if isinstance(result, (list, tuple)) and len(result) >= 5:
        num_loads = result[0]
        load_types = result[1] if result[1] else []
        load_names = result[2] if result[2] else []
        scale_factors = result[3] if result[3] else []
        ret = result[4]
        
        for i in range(num_loads):
            load = LoadCaseLoad(
                load_type=load_types[i] if i < len(load_types) else "Load",
                load_name=load_names[i] if i < len(load_names) else "",
                scale_factor=scale_factors[i] if i < len(scale_factors) else 1.0
            )
            loads.append(load)
        
        return loads, ret
    
    return [], -1


def set_static_linear_loads(
    model, 
    name: str, 
    loads: List[LoadCaseLoad]
) -> int:
    """
    设置线性静力工况的荷载数据
    
    Args:
        model: SapModel 对象
        name: 工况名称
        loads: 荷载列表
        
    Returns:
        0 表示成功
    """
    if not loads:
        return -1
    
    num_loads = len(loads)
    load_types = [load.load_type for load in loads]
    load_names = [load.load_name for load in loads]
    scale_factors = [load.scale_factor for load in loads]
    
    return model.LoadCases.StaticLinear.SetLoads(
        name, num_loads, load_types, load_names, scale_factors
    )



# -*- coding: utf-8 -*-
"""
loading - 荷载模式、工况、组合和质量源

SAP2000 术语:
    LoadPattern - 荷载模式 (如 DEAD, LIVE)
    LoadCase - 荷载工况/分析工况 (如 线性静力、模态、反应谱)
    LoadCombination - 荷载组合
    MassSource - 质量源 (动力分析质量来源)
"""

from .load_pattern import LoadPattern, LoadPatternType
from .load_case import (
    LoadCase,
    LoadCaseType,
    LoadCaseLoad,
    ModalSubType,
    TimeHistorySubType,
    DesignTypeOption,
    # 创建函数
    create_static_linear_case,
    create_static_nonlinear_case,
    create_modal_eigen_case,
    create_modal_ritz_case,
    create_response_spectrum_case,
    create_buckling_case,
    create_direct_history_linear_case,
    create_direct_history_nonlinear_case,
    create_modal_history_linear_case,
    create_modal_history_nonlinear_case,
    create_steady_state_case,
    create_psd_case,
    create_moving_load_case,
    create_hyperstatic_case,
    create_static_linear_multistep_case,
    create_static_nonlinear_multistep_case,
    create_staged_construction_case,
    # 荷载设置函数
    get_static_linear_loads,
    set_static_linear_loads,
)
from .load_combination import (
    # 枚举
    ComboCaseType,
    ComboType,
    # 函数
    add_combo,
    add_design_default_combos,
    change_combo_name,
    get_combo_count,
    get_combo_case_count,
    delete_combo,
    delete_combo_case,
    get_combo_name_list,
    get_combo_case_list,
    set_combo_case_list,
    get_combo_note,
    set_combo_note,
    get_combo_type,
    set_combo_type,
)
from .mass_source import MassSource, MassSourceLoad

__all__ = [
    # 荷载模式
    "LoadPattern",
    "LoadPatternType",
    # 荷载工况
    "LoadCase",
    "LoadCaseType",
    "LoadCaseLoad",
    "ModalSubType",
    "TimeHistorySubType",
    "DesignTypeOption",
    # 工况创建函数
    "create_static_linear_case",
    "create_static_nonlinear_case",
    "create_modal_eigen_case",
    "create_modal_ritz_case",
    "create_response_spectrum_case",
    "create_buckling_case",
    "create_direct_history_linear_case",
    "create_direct_history_nonlinear_case",
    "create_modal_history_linear_case",
    "create_modal_history_nonlinear_case",
    "create_steady_state_case",
    "create_psd_case",
    "create_moving_load_case",
    "create_hyperstatic_case",
    "create_static_linear_multistep_case",
    "create_static_nonlinear_multistep_case",
    "create_staged_construction_case",
    # 荷载设置函数
    "get_static_linear_loads",
    "set_static_linear_loads",
    # 荷载组合
    "ComboCaseType",
    "ComboType",
    "add_combo",
    "add_design_default_combos",
    "change_combo_name",
    "get_combo_count",
    "get_combo_case_count",
    "delete_combo",
    "delete_combo_case",
    "get_combo_name_list",
    "get_combo_case_list",
    "set_combo_case_list",
    "get_combo_note",
    "set_combo_note",
    "get_combo_type",
    "set_combo_type",
    # 质量源
    "MassSource",
    "MassSourceLoad",
]

# AI Agent 友好的 API 分类
LOADING_API_CATEGORIES = {
    "load_pattern": {
        "description": "荷载模式 (DEAD, LIVE 等)",
        "classes": ["LoadPattern"],
        "enums": ["LoadPatternType"],
    },
    "load_case": {
        "description": "荷载工况/分析工况",
        "classes": ["LoadCase", "LoadCaseLoad"],
        "enums": ["LoadCaseType", "ModalSubType", "TimeHistorySubType", "DesignTypeOption"],
        "functions": {
            "create": [
                "create_static_linear_case",
                "create_static_nonlinear_case",
                "create_modal_eigen_case",
                "create_modal_ritz_case",
                "create_response_spectrum_case",
                "create_buckling_case",
                "create_direct_history_linear_case",
                "create_direct_history_nonlinear_case",
                "create_modal_history_linear_case",
                "create_modal_history_nonlinear_case",
                "create_steady_state_case",
                "create_psd_case",
                "create_moving_load_case",
                "create_hyperstatic_case",
                "create_static_linear_multistep_case",
                "create_static_nonlinear_multistep_case",
                "create_staged_construction_case",
            ],
            "static_linear_loads": [
                "get_static_linear_loads",
                "set_static_linear_loads",
            ],
        },
    },
    "load_combination": {
        "description": "荷载组合",
        "classes": ["LoadCombination"],
    },
    "mass_source": {
        "description": "质量源 (动力分析质量来源)",
        "classes": ["MassSource", "MassSourceLoad"],
    },
}



# -*- coding: utf-8 -*-
"""
edge_constraint.py - 面单元边缘约束函数
对应 SAP2000 的 AreaObj 边缘约束相关 API
"""

from .enums import ItemType


def set_area_edge_constraint(
    model,
    area_name: str,
    constraint_exists: bool,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置面单元边缘约束
    
    边缘约束用于确保相邻面单元在边缘处的位移兼容性。
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        constraint_exists: 是否存在边缘约束
        item_type: 项目类型
        
    Returns:
        0 表示成功，非 0 表示失败
        
    Example:
        # 启用边缘约束
        set_area_edge_constraint(model, "1", True)
        
        # 禁用边缘约束
        set_area_edge_constraint(model, "1", False)
    """
    return model.AreaObj.SetEdgeConstraint(str(area_name), constraint_exists, int(item_type))


def get_area_edge_constraint(
    model,
    area_name: str
) -> bool:
    """
    获取面单元边缘约束状态
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        
    Returns:
        True 表示存在边缘约束，False 表示不存在
        
    Example:
        has_constraint = get_area_edge_constraint(model, "1")
        print(f"边缘约束: {'启用' if has_constraint else '禁用'}")
    """
    try:
        result = model.AreaObj.GetEdgeConstraint(str(area_name), False)
        if isinstance(result, (list, tuple)) and len(result) >= 2:
            return result[0]
    except Exception:
        pass
    return False


def enable_area_edge_constraint(
    model,
    area_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    启用面单元边缘约束
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        item_type: 项目类型
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    return set_area_edge_constraint(model, area_name, True, item_type)


def disable_area_edge_constraint(
    model,
    area_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    禁用面单元边缘约束
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        item_type: 项目类型
        
    Returns:
        0 表示成功，非 0 表示失败
    """
    return set_area_edge_constraint(model, area_name, False, item_type)


def has_area_edge_constraint(
    model,
    area_name: str
) -> bool:
    """
    检查面单元是否有边缘约束
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        
    Returns:
        True 表示有边缘约束，False 表示无
    """
    return get_area_edge_constraint(model, area_name)



# -*- coding: utf-8 -*-
"""
enums.py - 杆件相关枚举类型

包含 SAP2000 FrameObj API 使用的枚举

注意: 荷载相关枚举已移至 loads/frame_load.py
"""

from enum import IntEnum


class FrameType(IntEnum):
    """杆件类型"""
    BEAM = 1      # 梁
    COLUMN = 2    # 柱
    BRACE = 3     # 支撑
    TRUSS = 4     # 桁架
    OTHER = 5     # 其他


class FrameSectionType(IntEnum):
    """
    截面类型
    对应 SAP2000 PropFrame.GetTypeOAPI 返回值
    """
    I_SECTION = 1           # I形截面
    CHANNEL = 2             # 槽钢
    T_SECTION = 3           # T形截面
    ANGLE = 4               # 角钢
    DOUBLE_ANGLE = 5        # 双角钢
    BOX = 6                 # 箱形/方管
    PIPE = 7                # 圆管
    RECTANGULAR = 8         # 矩形
    CIRCLE = 9              # 圆形
    GENERAL = 10            # 通用截面
    DOUBLE_CHANNEL = 11     # 双槽钢
    AUTO = 12               # 自动选择
    SD = 13                 # 截面设计器
    VARIABLE = 14           # 变截面
    JOIST = 15              # 托梁
    BRIDGE = 16             # 桥梁截面
    COLD_C = 17             # 冷弯C型钢
    COLD_2C = 18            # 冷弯双C型钢
    COLD_Z = 19             # 冷弯Z型钢
    COLD_L = 20             # 冷弯角钢
    COLD_2L = 21            # 冷弯双角钢
    COLD_HAT = 22           # 冷弯帽形钢
    BUILTUP_I_COVERPLATE = 23  # 组合I形盖板
    PCC_GIRDER_I = 24       # 预制混凝土I梁
    PCC_GIRDER_U = 25       # 预制混凝土U梁
    BUILTUP_I_HYBRID = 26   # 组合I形混合
    BUILTUP_U_HYBRID = 27   # 组合U形混合
    PCC_GIRDER_SUPER_T = 41 # 预制混凝土超级T梁
    COLD_BOX = 42           # 冷弯箱形
    COLD_I = 43             # 冷弯I形
    COLD_PIPE = 44          # 冷弯圆管
    COLD_T = 45             # 冷弯T形
    TRAPEZOIDAL = 46        # 梯形


class FrameReleaseType(IntEnum):
    """杆件端部释放类型（便捷枚举）"""
    BOTH_FIXED = 0    # 两端固定
    I_END_HINGED = 1  # I端铰接
    J_END_HINGED = 2  # J端铰接
    BOTH_HINGED = 3   # 两端铰接


class ItemType(IntEnum):
    """
    eItemType 枚举
    用于批量操作
    """
    OBJECT = 0           # 单个对象
    GROUP = 1            # 组内所有对象
    SELECTED_OBJECTS = 2 # 所有选中的对象


# 截面类型到中文名称的映射 (便于 AI Agent 理解)
SECTION_TYPE_NAMES = {
    FrameSectionType.I_SECTION: "I形截面",
    FrameSectionType.CHANNEL: "槽钢",
    FrameSectionType.T_SECTION: "T形截面",
    FrameSectionType.ANGLE: "角钢",
    FrameSectionType.DOUBLE_ANGLE: "双角钢",
    FrameSectionType.BOX: "箱形/方管",
    FrameSectionType.PIPE: "圆管",
    FrameSectionType.RECTANGULAR: "矩形",
    FrameSectionType.CIRCLE: "圆形",
    FrameSectionType.GENERAL: "通用截面",
    FrameSectionType.DOUBLE_CHANNEL: "双槽钢",
    FrameSectionType.AUTO: "自动选择",
    FrameSectionType.SD: "截面设计器",
    FrameSectionType.VARIABLE: "变截面",
}


# 端部释放预设 (I端, J端)
RELEASE_PRESETS = {
    FrameReleaseType.BOTH_FIXED: (
        (False, False, False, False, False, False),
        (False, False, False, False, False, False)
    ),
    FrameReleaseType.I_END_HINGED: (
        (False, False, False, False, True, True),
        (False, False, False, False, False, False)
    ),
    FrameReleaseType.J_END_HINGED: (
        (False, False, False, False, False, False),
        (False, False, False, False, True, True)
    ),
    FrameReleaseType.BOTH_HINGED: (
        (False, False, False, False, True, True),
        (False, False, False, False, True, True)
    ),
}



# -*- coding: utf-8 -*-
"""
group.py - 面单元组分配函数
对应 SAP2000 的 AreaObj 组分配相关 API

SAP2000 API:
- AreaObj.SetGroupAssign(Name, GroupName, Remove, ItemType)
- AreaObj.GetGroupAssign(Name, NumberGroups, Groups[])
"""

from typing import List, Optional

from .enums import ItemType


def set_area_group(
    model,
    area_name: str,
    group_name: str,
    remove: bool = False,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    设置面单元组分配
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        group_name: 组名称 (必须已存在)
        remove: False=添加到组, True=从组移除
        item_type: 操作范围
    
    Returns:
        0 表示成功，非 0 表示失败
    
    Example:
        # 将面单元添加到组
        set_area_group(model, "1", "Slabs")
        
        # 从组移除面单元
        set_area_group(model, "1", "Slabs", remove=True)
    """
    return model.AreaObj.SetGroupAssign(str(area_name), group_name, remove, int(item_type))


def add_area_to_group(
    model,
    area_name: str,
    group_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    将面单元添加到组
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        group_name: 组名称 (必须已存在)
        item_type: 操作范围
    
    Returns:
        0 表示成功，非 0 表示失败
    
    Example:
        add_area_to_group(model, "1", "Slabs")
    """
    return set_area_group(model, area_name, group_name, False, item_type)


def remove_area_from_group(
    model,
    area_name: str,
    group_name: str,
    item_type: ItemType = ItemType.OBJECT
) -> int:
    """
    从组移除面单元
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        group_name: 组名称
        item_type: 操作范围
    
    Returns:
        0 表示成功，非 0 表示失败
    
    Example:
        remove_area_from_group(model, "1", "Slabs")
    """
    return set_area_group(model, area_name, group_name, True, item_type)


def get_area_groups(
    model,
    area_name: str
) -> Optional[List[str]]:
    """
    获取面单元所属组
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
    
    Returns:
        组名称列表，失败返回 None
    
    Example:
        groups = get_area_groups(model, "1")
        if groups:
            print(f"面单元所属组: {groups}")
    """
    try:
        result = model.AreaObj.GetGroupAssign(str(area_name), 0, [])
        if isinstance(result, (list, tuple)) and len(result) >= 3:
            num_groups = result[0]
            groups = result[1]
            if num_groups > 0 and groups:
                return list(groups)
    except Exception:
        pass
    return None


def is_area_in_group(
    model,
    area_name: str,
    group_name: str
) -> bool:
    """
    检查面单元是否在指定组中
    
    Args:
        model: SapModel 对象
        area_name: 面单元名称
        group_name: 组名称
    
    Returns:
        True=在组中, False=不在组中
    
    Example:
        if is_area_in_group(model, "1", "Slabs"):
            print("面单元在 Slabs 组中")
    """
    groups = get_area_groups(model, area_name)
    if groups:
        return group_name in groups
    return False


def add_areas_to_group(
    model,
    area_names: List[str],
    group_name: str
) -> int:
    """
    批量将面单元添加到组
    
    Args:
        model: SapModel 对象
        area_names: 面单元名称列表
        group_name: 组名称 (必须已存在)
    
    Returns:
        0 表示全部成功，非 0 表示有失败
    
    Example:
        add_areas_to_group(model, ["1", "2", "3"], "Slabs")
    """
    ret = 0
    for name in area_names:
        result = add_area_to_group(model, name, group_name)
        if result != 0:
            ret = result
    return ret


def remove_areas_from_group(
    model,
    area_names: List[str],
    group_name: str
) -> int:
    """
    批量从组移除面单元
    
    Args:
        model: SapModel 对象
        area_names: 面单元名称列表
        group_name: 组名称
    
    Returns:
        0 表示全部成功，非 0 表示有失败
    
    Example:
        remove_areas_from_group(model, ["1", "2", "3"], "Slabs")
    """
    ret = 0
    for name in area_names:
        result = remove_area_from_group(model, name, group_name)
        if result != 0:
            ret = result
    return ret



# -*- coding: utf-8 -*-
"""
文件操作工具 - 保存、解锁、新建、打开模型

重构: 复用 PySap2000.file 模块（如果存在）
"""

from langchain.tools import tool

from .base import get_sap_model, success_response, error_response, safe_sap_call


@tool
@safe_sap_call
def save_model() -> str:
    """保存当前模型。"""
    model = get_sap_model()
    ret = model.File.Save()
    
    if ret == 0:
        filename = model.GetModelFilename(False) or "未命名"
        return success_response("保存成功", 文件名=filename)
    return error_response("保存失败")


@tool
@safe_sap_call
def unlock_model() -> str:
    """解锁模型，允许修改。"""
    model = get_sap_model()
    model.SetModelIsLocked(False)
    return success_response("模型已解锁")


@tool
@safe_sap_call
def new_model(units: int = 6) -> str:
    """
    新建空白模型。
    
    Args:
        units: 单位制 (1=lb_in_F, 2=lb_ft_F, 3=kip_in_F, 4=kip_ft_F, 
               5=kN_mm_C, 6=kN_m_C, 7=kgf_mm_C, 8=kgf_m_C, 9=N_mm_C, 10=N_m_C)
    """
    model = get_sap_model()
    
    # 初始化新模型
    ret = model.InitializeNewModel(units)
    if ret != 0:
        return error_response("初始化新模型失败")
    
    # 创建空白模型
    ret = model.File.NewBlank()
    
    if ret == 0:
        unit_names = {
            1: "lb_in_F", 2: "lb_ft_F", 3: "kip_in_F", 4: "kip_ft_F",
            5: "kN_mm_C", 6: "kN_m_C", 7: "kgf_mm_C", 8: "kgf_m_C",
            9: "N_mm_C", 10: "N_m_C"
        }
        return success_response(
            "新模型已创建",
            单位制=unit_names.get(units, f"类型{units}")
        )
    return error_response("创建新模型失败")


@tool
@safe_sap_call
def open_model(file_path: str) -> str:
    """
    打开模型文件。
    
    Args:
        file_path: 模型文件完整路径 (.sdb 文件)
    """
    model = get_sap_model()
    ret = model.File.OpenFile(file_path)
    
    if ret == 0:
        return success_response("模型已打开", 文件=file_path)
    return error_response(f"打开模型失败: {file_path}")
